<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers;

use WPSEC\Helpers\{
    NotifyAdmin,
    OutOfBondEmail,
};
use WPSEC\Controllers\{
    Modules\TwoFASettings,
    Modules\LoginAttempts,
    Modules\RememberMe,
};
use WPSEC\Views\LoginForms;
use WPSEC\Validators\Validator;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Controllers\LoginCheck' ) ) {
    /**
     * Checks user input after user login
     *
     * @since 1.0.0
     */
    class LoginCheck {

        /**
         * Checks the user login credentials
         *
         * TODO: change the loging in user check - set proper priorities
         *
         * 1. Checks if user is locked
         * 2. Checks if module is enabled
         * 3. Checks if user is set and not error
         * 4. Checks login attempts
         * 5. Clear and login
         *
         * @since 1.0.0
         *
         * @param \WP_USER $user - user object or error.
         * @param string   $username - the username currently tries to login.
         * @param string   $password - the password for currently trying to login user.
         *
         * @return \WP_User|\WP_Error
         */
        public static function check( $user, $username, $password ) {

            if ( (bool) \WPSEC\Controllers\Modules\LoginAttempts::getGlobalSettingsValue()
                &&
                (bool) \WPSEC\Controllers\Modules\TwoFASettings::getGlobalSettingsValue() ) {
                    return $user;
            }

            if ( (bool) \WPSEC\Controllers\Modules\TwoFASettings::getGlobalSettingsValue() || User::isTwoFaUserExcluded( $username ) ) {
                if ( (bool) \WPSEC\Controllers\Modules\LoginAttempts::getGlobalSettingsValue() ) {
                    return $user;
                }
            }

            /**
             * Checks if user is locked out and if the module is enabled
             */
            if ( User::isLocked( $username ) && ! (bool) \WPSEC\Controllers\Modules\LoginAttempts::getGlobalSettingsValue() ) {

                \wp_clear_auth_cookie();

                $error = new \WP_Error(
                    'authentication_failed',
                    __( '<strong>Error</strong>: Too soon.', 'secured-wp' )
                );
                \do_action( 'wp_login_failed', $username, $error );

                return $error;
            }

            if ( null === $user || \is_wp_error( $user ) ) {
                if ( ! (bool) \WPSEC\Controllers\Modules\LoginAttempts::getGlobalSettingsValue() ) {
                    $userTriedToLogIn = \get_user_by( 'login', $username );

                    if ( $userTriedToLogIn ) {

                        LoginAttempts::increaseLoginAttempts( $userTriedToLogIn );

                        if (
                        LoginAttempts::getLoginAttempts( $userTriedToLogIn ) > LoginAttempts::getAllowedAttempts() ) {

                            User::lockUser( $username );

                            NotifyAdmin::sendNotificationEmail( $userTriedToLogIn, LoginAttempts::getLoginAttempts( $userTriedToLogIn ) );

                            $error = new \WP_Error(
                                'authentication_failed',
                                __( '<strong>Error</strong>: Too many attempts.', 'secured-wp' )
                            );
                            \do_action( 'wp_login_failed', $username, $error );

                            return $error;
                        }
                    }
                }
            } elseif ( ! \is_wp_error( $user ) ) {
                LoginAttempts::clearLoginAttempts( $user );
                if ( ! (bool) \WPSEC\Controllers\Modules\TwoFASettings::getGlobalSettingsValue() && TwoFASettings::isTotpEnabled() && ! User::isTwoFaUserExcluded( $username ) ) {
                    LoginForms::loginTotp( '', $user );

                    exit();
                }
            }

            return $user;
        }

        /**
         * Checks and validates 2FA login
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function loginValidateTwoFa() {
            if ( ! isset( $_POST[ LoginForms::getLoginNonceName() ] ) ) {
                return;
            }

            if (
                ! isset( $_POST[ LoginForms::getLoginNonceName() ] )
                || ! \wp_verify_nonce( \sanitize_text_field( \wp_unslash( $_POST[ LoginForms::getLoginNonceName() ] ) ), LoginForms::getLoginNonceName() )
                ) {
                \wp_nonce_ays( __( 'Non Valid nonce' ) );
            }

            $userID     = ( isset( $_REQUEST['2fa-auth-id'] ) ) ? (int) $_REQUEST['2fa-auth-id'] : false;
            $authCode   = ( isset( $_REQUEST['authcode'] ) ) ? (string) \sanitize_text_field( \wp_unslash( $_REQUEST['authcode'] ) ) : false;
            $redirectTo = ( isset( $_REQUEST['redirect_to'] ) ) ? (string) \esc_url_raw( \sanitize_text_field( \wp_unslash( $_REQUEST['redirect_to'] ) ) ) : '';

            if ( $userID && $authCode ) {
                if ( User::validateTotpAuthentication( $authCode, $userID ) ) {
                    \wp_set_auth_cookie( User::getUser()->ID );

                    LoginForms::interimLogin();

                    // Check if user has any roles/caps set - if doesn't, we know its a "network" user.
                    if ( \is_multisite() &&
                        ! \get_active_blog_for_user( User::getUser()->ID ) &&
                        empty( User::getUser()->caps ) ) {
                        $redirectTo = \user_admin_url();
                    } else {
                        $redirectTo = \apply_filters(
                            'login_redirect',
                            $redirectTo,
                            $redirectTo,
                            User::getUser()
                        );
                    }

                    /** Sets the TOTP status of the user */
                    if ( ! User::isTotpUserEnabled() ) {
                        User::enableTotp();
                    }

                    \wp_safe_redirect( $redirectTo );

                    exit();
                } else {
                    $error = __( 'Invalid code provided', 'secured-wp' );
                    LoginForms::loginTotp( $error );

                    exit();
                }
            }
        }

        /**
         * Validates the link from the WP form
         *
         * @hook login_form_confirm_oob
         *
         * @return void
         *
         * @since
         */
        public static function loginValidateOob() {
            self::checkOOBAndLogin( true );
        }

        /**
         * Send remember me cookie - RememberMe call
         * Sets the users logged in devices (adds new one if necessarily)
         *
         * @hook set_logged_in_cookie
         *
         * @since 1.0.0
         *
         * @return void
         *
         * @SupressWarnings(PHPMD.Superglobals)
         */
        public static function setRememberMe() {

            if ( isset( $_POST['rememberme'] ) && ! empty( $_POST['rememberme'] ) ) {
                RememberMe::setRememberMe( User::getUser()->ID );
                User::setLoggedInDevice( RememberMe::getDevice() );
            }
        }

        /**
         * Checks Out of bond email link and logging the user if passed
         *
         * @param bool $secondPass - first pass comes from e-mail (link), second pass comes from WP form itself.
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function checkOOBAndLogin( bool $secondPass = false ) {
            // No logged user? continue if so.
            if ( ! User::isCurrentlyLogged() ) {
                $params = $_GET; // @codingStandardsIgnoreLine - Nonce check is made few lines below
                if ( $secondPass ) {
                    $params = $_POST; // @codingStandardsIgnoreLine - Nonce check is made few lines below
                }
                // is there otp key?
                if ( isset( $params['wps_otp'] ) && ! empty( $params['wps_otp'] ) ) {
                    // is there user id?
                    if ( isset( $params['user_id'] ) && ! empty( $params['user_id'] ) ) {

                        // are the formats correct.
                        if ( Validator::filterValidate( \sanitize_text_field( \wp_unslash( $params['user_id'] ) ), 'int' ) &&
                        Validator::validateNumber( \sanitize_text_field( \wp_unslash( $params['wps_otp'] ) ) ) ) {
                            $userId = (int) \sanitize_text_field( \wp_unslash( $params['user_id'] ) );
                            $wpsOtp = (string) \sanitize_text_field( \wp_unslash( $params['wps_otp'] ) );

                            $nonceName = OutOfBondEmail::getNonceName();

                            if ( isset( $params[ $nonceName ] ) && ! empty( $params[ $nonceName ] ) ) {
                                if ( ! \wp_verify_nonce( \sanitize_text_field( \wp_unslash( $params[ $nonceName ] ) ), OutOfBondEmail::getNonceNamePrefix() . $userId ) ) {
                                    exit();
                                }

                                // check transient.
                                $dataTimeout = \get_option( '_transient_timeout_' . OutOfBondEmail::getTransientPrefix() . ( (string) $userId ) );
                                if ( $dataTimeout > time() ) {
                                    if ( OutOfBondEmail::validateUserOOB( $userId, $wpsOtp ) ) {

                                        if ( ! $secondPass ) {
                                            LoginForms::loginOob( '', $userId );
                                            exit();
                                        }
                                        \wp_clear_auth_cookie();
                                        OutOfBondEmail::deleteTransient( $userId );
                                        \wp_set_current_user( $userId );
                                        \wp_set_auth_cookie( $userId );

                                        if ( ! isset( $params['redirect_to'] ) || empty( $params['redirect_to'] ) ) {
                                            $redirectTo = \user_admin_url();
                                        } else {
                                            $redirectTo = \esc_url_raw( \wp_unslash( $params['redirect_to'] ) );
                                        }

                                        \wp_safe_redirect( $redirectTo );
                                        exit();
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        /**
         * Clears the stored cookie and transient from the system
         * Clean the logged in devices from the user array
         *
         * @hook wp_logout
         *
         * @since 1.0.0
         *
         * @param [type] $userId - the id of the user.
         *
         * @return void
         */
        public static function logOut( $userId ) {
            RememberMe::clearRememberMe( $userId );
            User::cleanLoggedInDevices( $userId );
        }
    }
}
