<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers\Modules;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use WPSEC\Helpers\WPHelper;

if ( ! class_exists( 'WPSEC\Controllers\Modules\BaseModule' ) ) {
    /**
     * Base module class
     *
     * @since 1.0.0
     */
    abstract class BaseModule {

        /**
         * Global settings name - declared in the plugin main file
         * That must be always set (overridden) in the extending class
         *
         * @since 1.0.0
         *
         * @var string
         */
        protected static $globalSettingName;

        /**
         * Is that module (this is abstract class) enabled?
         *
         * @since 1.0.0
         *
         * @var null|bool
         */
        protected static $moduleEnabled;

        /**
         * Holds the name of the give module
         *
         * @var string
         *
         * @since 1.0.0
         */
        protected static $moduleName;

        /**
         * Saves global option for given module
         *
         * @since 1.0.0
         *
         * @param mixed  $value - the value from the given option.
         * @param string $blogId - blog id for the option.
         *
         * @return void
         */
        public static function saveGlobalEnabled( $value, $blogId = '' ) {
            WPHelper::setOption( static::$globalSettingName, $value, '', $blogId );
            static::$moduleEnabled = filter_var( $value, FILTER_VALIDATE_BOOLEAN );
        }

        /**
         * Returns the name of the global setting for the given module
         *
         * @since 1.0.0
         *
         * @return string
         */
        public static function getGlobalSettingsName(): string {
            return static::$globalSettingName;
        }

        /**
         * Returns global settings module status - enabled / disabled
         *
         * @since 1.0.0
         *
         * @param string $blogId - blog id for the option.
         *
         * @return boolean
         */
        public static function getGlobalSettingsValue( $blogId = '' ): bool {
            if ( null === static::$moduleEnabled ) {
                static::$moduleEnabled =
                WPHelper::getOption(
                    static::$globalSettingName,
                    false,
                    '',
                    $blogId
                );

                static::$moduleEnabled = filter_var( static::$moduleEnabled, FILTER_VALIDATE_BOOLEAN );
            }

            return static::$moduleEnabled;
        }

        /**
         * Setter for the module name
         *
         * @param string $moduleName - module name.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function setModuleName( string $moduleName = '' ) {
            static::$moduleName = $moduleName;
        }

        /**
         * Returns module name
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getModuleName(): string {
            if ( null === self::$moduleName ) {
                static::setModuleName();
            }
            return static::$moduleName;
        }

        /**
         * Returns info for the module
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function getInfo(): array {
            return [
                'name'     => self::getModuleName(),
                'disabled' => self::getGlobalSettingsValue(),
            ];
        }

        /**
         * Returns short description about the module
         *
         * @return string
         *
         * @since 1.0.0
         */
        abstract public static function getShortDescription(): string;

        /**
         * Deletes all the module related data
         *
         * @return bool
         *
         * @since 1.0.0
         */
        abstract public static function deleteModuleData(): bool;
    }
}
