<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers\Modules;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use WPSEC\Controllers\User;
use WPSEC\Helpers\WPHelper;

if ( ! class_exists( 'WPSEC\Controllers\Modules\Login' ) ) {
    /**
     * Masks original WP login
     * Redirects to new login slug
     *
     * @since 1.0.0
     */
    class Login extends BaseModule {

        /**
         * Global setting name - stored the global value for enable / disable module
         *
         * @var string
         */
        protected static $globalSettingName = WPSEC_PLUGIN_SECURED_LOGIN_MODULE_DISABLED;

        /**
         * Is the module enabled or not
         *
         * @var bool
         *
         * @since 1.0.0
         */
        protected static $moduleEnabled;

        /**
         * Holds the name of the give module
         *
         * @since 1.0.0
         *
         * @var string
         */
        protected static $moduleName;

        /**
         * Flag for user request - tries to login?
         *
         * @since 1.0.0
         *
         * @var boolean
         */
        private static $wpLogin = false;

        /**
         * Init all the hooks related to login redirection
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function init() {

            if ( WPHelper::isMultisite() && ! function_exists( 'is_plugin_active_for_network' ) || ! function_exists( 'is_plugin_active' ) ) {
                require_once ABSPATH . '/wp-admin/includes/plugin.php';
            }

            if ( WPHelper::isMultisite() ) {
                \add_action( 'wp_before_admin_bar_render', [ __CLASS__, 'modifyMysitesMenu' ], 999 );
            }

            \add_action( 'plugins_loaded', [ __CLASS__, 'pluginsLoaded' ], 9999 );
            \add_action( 'wp_loaded', [ __CLASS__, 'wpLoaded' ] );
            \add_action( 'setup_theme', [ __CLASS__, 'setupTheme' ], 1 );

            \add_filter( 'site_url', [ __CLASS__, 'siteUrl' ], 10, 4 );
            \add_filter( 'network_site_url', [ __CLASS__, 'networkSiteUrl' ], 10, 3 );
            \add_filter( 'site_option_welcome_email', [ __CLASS__, 'welcomeEmail' ] );

            \remove_action( 'template_redirect', 'wp_redirect_admin_locations', 1000 );
            \add_action( 'template_redirect', [ __CLASS__, 'redirectExportData' ] );
            \add_filter( 'login_url', [ __CLASS__, 'loginUrl' ], 10, 3 );

            \add_filter( 'user_request_action_email_content', [ __CLASS__, 'userRequestActionEmailContent' ], 999, 2 );

            \add_filter( 'site_status_tests', [ __CLASS__, 'siteStatusTests' ] );
        }

        /**
         * Removes loopback requests tests
         *
         * @since 1.0.0
         *
         * @param array $tests - status test array.
         *
         * @return array
         */
        public static function siteStatusTests( array $tests ): array {
            unset( $tests['async']['loopback_requests'] );

            return $tests;
        }

        /**
         * Filters the text of the email sent to user
         *
         * @since 1.0.0
         *
         * @param string $emailText - text for the email.
         * @param array  $emailData - email data vars.
         *
         * @see user_request_action_email_content filter
         *
         * @return string
         */
        public static function userRequestActionEmailContent( $emailText, $emailData ): string {
            $emailText = str_replace( '###CONFIRM_URL###', \esc_url_raw( str_replace( self::newLoginSlug() . '/', 'wp-login.php', $emailData['confirm_url'] ) ), $emailText );

            return $emailText;
        }

        /**
         * Should trailing slashes be used or not
         *
         * @since 1.0.0
         *
         * @return boolean
         */
        private static function useTrailingSlashes(): bool {

            return ( '/' === substr( WPHelper::getOption( 'permalink_structure', '' ), - 1, 1 ) );

        }

        /**
         * Adds trailing slash if necessary
         *
         * @since 1.0.0
         *
         * @param string $string - adds trailing slash based on the selection.
         *
         * @return string
         */
        private static function useTrailingslashit( string $string ): string {

            return self::useTrailingSlashes() ? \trailingslashit( $string ) : \untrailingslashit( $string );

        }

        /**
         * Loads the WP template
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function wpTemplateLoader() {
            global $pagenow;

            $pagenow = 'index.php'; // @codingStandardsIgnoreLine

            if ( ! defined( 'WP_USE_THEMES' ) ) {

                define( 'WP_USE_THEMES', true );

            }

            \wp();

            require_once ABSPATH . WPINC . '/template-loader.php';

            \wp_die();
        }

        /**
         * Modifies Multisite My Sites menu login links
         *
         * @since 1.0.0
         *
         * @return void
         *
         * @SuppressWarnings(PHPMD.CamelCaseVariableName)
         */
        public static function modifyMysitesMenu() {
            global $wp_admin_bar;

            $all_toolbar_nodes = $wp_admin_bar->get_nodes();

            foreach ( $all_toolbar_nodes as $node ) {
                if ( preg_match( '/^blog-(\d+)(.*)/', $node->id, $matches ) ) {
                    $blog_id = $matches[1];
                    if ( $login_slug = self::newLoginSlug( $blog_id ) ) {
                        if ( ! $matches[2] || '-d' === $matches[2] ) {
                            $args       = $node;
                            $old_href   = $args->href;
                            $args->href = preg_replace( '/wp-admin\/$/', "$login_slug/", $old_href );
                            if ( $old_href !== $args->href ) {
                                $wp_admin_bar->add_node( $args );
                            }
                        } elseif ( strpos( $node->href, '/wp-admin/' ) !== false ) {
                            $wp_admin_bar->remove_node( $node->id );
                        }
                    }
                }
            }
        }

        /**
         * Updates the new login slug
         *
         * @since 1.0.0
         *
         * @param mixed $value - value to be stored.
         * @param mixed $blogId - WP blog id.
         *
         * @return void
         */
        public static function updateNewLoginSlug( $value, $blogId = '' ) {
            WPHelper::setOption( WPSEC_PLUGIN_SECURED_NEW_LOGIN_REDIRECT_VAR_NAME, $value, '', $blogId );
        }

        /**
         * Removes the new login slug (restores defaults)
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - WP blog id.
         *
         * @return void
         */
        public static function deleteNewLoginSlug( $blogId = '' ) {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_NEW_LOGIN_REDIRECT_VAR_NAME, $blogId );
        }

        /**
         * Extracts new login slug and returns it
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - WP blog id.
         *
         * @return string
         */
        public static function newLoginSlug( $blogId = '' ): string {
            return WPHelper::getOption(
                WPSEC_PLUGIN_SECURED_NEW_LOGIN_REDIRECT_VAR_NAME,
                WPSEC_PLUGIN_SECURED_NEW_LOGIN_REDIRECT_VAR_DEFAULT,
                '',
                $blogId
            );
        }

        /**
         * Returns slug if someone tries to access the default wp-login.php
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - WP blog id.
         *
         * @return string
         */
        public static function newRedirectSlug( $blogId = '' ): string {
            return WPHelper::getOption(
                WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_NAME,
                WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_DEFAULT,
                '',
                $blogId
            );
        }

        /**
         * Updates the new redirect slug
         *
         * @since 1.0.0
         *
         * @param mixed $value - new value.
         * @param mixed $blogId - WP blog id.
         *
         * @return void
         */
        public static function updateNewRedirectSlug( $value, $blogId = '' ) {
            WPHelper::setOption( WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_NAME, $value, '', $blogId );
        }

        /**
         * Deletes the new redirection option
         *
         * @since 1.0.0
         *
         * @param string $blogId - WP blog id.
         *
         * @return void
         */
        public static function deleteNewRedirectSlug( $blogId = '' ) {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_NAME, $blogId );
        }

        /**
         * Returns new login url
         *
         * @since 1.0.0
         *
         * @param mixed $scheme - the URL scheme.
         *
         * @return string
         */
        public static function newLoginUrl( $scheme = null ): string {

            $url = \home_url( '/', $scheme );

            if ( \get_option( 'permalink_structure' ) ) {
                return self::useTrailingslashit( $url . self::newLoginSlug() );
            } else {
                return $url . '?' . self::newLoginSlug();
            }

        }

        /**
         * Builds URL to redirect to newly selected login location
         *
         * @since 1.0.0
         *
         * @param string $scheme - the URL scheme.
         *
         * @return string
         */
        public static function newRedirectUrl( $scheme = null ) {

            if ( \get_option( 'permalink_structure' ) ) {
                return self::useTrailingslashit( home_url( '/', $scheme ) . self::newRedirectSlug() );
            } else {
                return \home_url( '/', $scheme ) . '?' . self::newRedirectSlug();
            }
        }

        /**
         * Fires on template redirect - @see 'template_redirect'
         *
         * @hook 'template_redirect'
         *
         * @return void
         *
         * @since 1.0.0
         *
         * @SuppressWarnings(PHPMD.ExitExpression)
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function redirectExportData() {
            if ( ! empty( $_GET ) && isset( $_GET['action'] ) && 'confirmaction' === $_GET['action'] && isset( $_GET['request_id'] ) && isset( $_GET['confirm_key'] ) ) {
                $requestId = (int) $_GET['request_id'];
                $key       = \sanitize_text_field( \wp_unslash( $_GET['confirm_key'] ) );
                $result    = \wp_validate_user_request_key( $requestId, $key );
                if ( ! \is_wp_error( $result ) ) {
                    \wp_safe_redirect(
                        \add_query_arg(
                            [
                                'action'      => 'confirmaction',
                                'request_id'  => $requestId,
                                'confirm_key' => $key,
                            ],
                            self::newLoginUrl()
                        )
                    );
                    exit();
                }
            }
        }

        /**
         * Fires right after all the plugins are loaded
         * checks the current URL and sets the flag if
         * redirection is needed later on
         *
         * @since 1.0.0
         *
         * @return void
         *
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function pluginsLoaded() {
            global $pagenow;

            if ( ! WPHelper::isMultisite()
                && ( strpos( rawurldecode( $_SERVER['REQUEST_URI'] ), 'wp-signup' ) !== false // @codingStandardsIgnoreLine
                || strpos( rawurldecode( $_SERVER['REQUEST_URI'] ), 'wp-activate' ) !== false ) // @codingStandardsIgnoreLine
                && \apply_filters( 'WPSEC_hide_login_signup_enable', false ) === false ) { // @codingStandardsIgnoreLine

                \wp_die( \esc_html__( 'This feature is not enabled.', 'secured-wp' ) );
            }

            $request = parse_url( rawurldecode( $_SERVER['REQUEST_URI'] ) ); // @codingStandardsIgnoreLine

            if ( ( strpos( rawurldecode( $_SERVER['REQUEST_URI'] ), 'wp-login.php' ) !== false // @codingStandardsIgnoreLine
                || ( isset( $request['path'] )
                && \untrailingslashit( $request['path'] ) === \site_url( 'wp-login', 'relative' ) ) )
                && ! \is_admin() ) {

                if ( 'checkemail=registered' === $request['query'] ) {
					$pagenow = 'wp-login.php'; // @codingStandardsIgnoreLine
                } else {
                    self::setWPLogin( true );

                    $_SERVER['REQUEST_URI'] = self::useTrailingslashit( '/' . str_repeat( '-/', 10 ) );

                    $pagenow = 'index.php'; // @codingStandardsIgnoreLine
                }
            } elseif ( ( isset( $request['path'] ) && \untrailingslashit( $request['path'] ) === \home_url( self::newLoginSlug(), 'relative' ) )
                || ( ! \get_option( 'permalink_structure' )
                && isset( $_GET[ self::newLoginSlug() ] )
                && empty( $_GET[ self::newLoginSlug() ] ) ) ) {

                $pagenow = 'wp-login.php'; // @codingStandardsIgnoreLine

            } elseif ( ( strpos( rawurldecode( $_SERVER['REQUEST_URI'] ), 'wp-register.php' ) !== false // @codingStandardsIgnoreLine
                || ( isset( $request['path'] )
                && \untrailingslashit( $request['path'] ) === \site_url( 'wp-register', 'relative' ) ) )
                && ! \is_admin() ) {

                self::setWpLogin( true );

                $_SERVER['REQUEST_URI'] = self::useTrailingslashit( '/' . str_repeat( '-/', 10 ) );

                $pagenow = 'index.php'; // @codingStandardsIgnoreLine
            }
        }

        /**
         * IF the user is not logged and tries to customize
         * the theme - returns error
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function setupTheme() {
            global $pagenow;

            if ( ! User::isCurrentlyLogged() && 'customize.php' === $pagenow ) {
                \wp_die( \esc_html__( 'Nothing to see here', 'secured-wp' ), 403 );
            }
        }

        /**
         * Triggers right after the WP is loaded and ready
         * Checks current page and if the user is logged in
         * If the call is from the new login url and flag is set
         * makes redirection
         *
         * @since 1.0.0
         *
         * @return void
         *
         * @SuppressWarnings(PHPMD.ExitExpression)
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function wpLoaded() {
            global $pagenow;

            $request = parse_url( rawurldecode( $_SERVER['REQUEST_URI'] ) ); // @codingStandardsIgnoreLine

            if ( ! ( isset( $_GET['action'] ) && 'postpass' === $_GET['action'] && isset( $_POST['post_password'] ) ) ) { // @codingStandardsIgnoreLine

                if ( ( self::getWpLogin() && ! User::isCurrentlyLogged() ) || ( \is_admin() && ! User::isCurrentlyLogged() && ! defined( 'DOING_AJAX' ) && 'admin-post.php' !== $pagenow && '/wp-admin/options.php' !== $request['path'] ) ) {
                    \wp_safe_redirect( self::newRedirectUrl() );
                    die(); // @codingStandardsIgnoreLine
                }

                if ( 'wp-login.php' === $pagenow
                && self::useTrailingslashit( $request['path'] ) !== $request['path']
                && \get_option( 'permalink_structure' ) ) {

                    \wp_safe_redirect(
                        self::useTrailingslashit( self::newLoginUrl() )
                        . ( ! empty( $_SERVER['QUERY_STRING'] ) ? '?' . \wp_unslash( $_SERVER['QUERY_STRING'] ) : '' )
                    );

                    die();

                } elseif ( self::getWpLogin() ) {

                    if ( ( $referer = \wp_get_referer() )
                    && strpos( $referer, 'wp-activate.php' ) !== false
                    && ( $referer = parse_url( $referer ) )
                    && ! empty( $referer['query'] ) ) {

                        parse_str( $referer['query'], $referer );

                        @require_once \WPINC . '/ms-functions.php';

                        if ( ! empty( $referer['key'] )
                        && ( $result = \wpmu_activate_signup( $referer['key'] ) )
                        && \is_wp_error( $result )
                        && ( $result->get_error_code() === 'already_active'
                        || $result->get_error_code() === 'blog_taken' ) ) {

                            \wp_safe_redirect(
                                self::newLoginUrl()
                                . ( ! empty( $_SERVER['QUERY_STRING'] ) ? '?' . \wp_unslash( $_SERVER['QUERY_STRING'] ) : '' )
                            );

                            die();
                        }
                    }

                    self::wpTemplateLoader();

                } elseif ( 'wp-login.php' === $pagenow ) {

                    $redirectTo = \admin_url();

                    if ( User::isCurrentlyLogged() ) {
                        if ( ! isset( $_REQUEST['action'] ) ) {
                            $loggedInRedirect = $redirectTo;
                            \wp_safe_redirect( $loggedInRedirect );

                            die();
                        }
                    }

                    @require_once ABSPATH . 'wp-login.php';

                    die();
                }
            }
        }

        /**
         * Checks the site url and changes it if necessary
         *
         * @since 1.0.0
         *
         * @param string $url - the url of the site.
         * @param string $path - currently not in use.
         * @param string $scheme - the URL scheme.
         * @param string $blogId - WP blog id.
         *
         * @return string
         */
        public static function siteUrl( $url, $path, $scheme, $blogId ) {

            return self::filterWpLoginPhp( $url, $scheme );
        }

        /**
         * Checks the network site url and changes it if necessary
         *
         * @since 1.0.0
         *
         * @param string $url - the url of the site.
         * @param string $path - currently not in use.
         * @param string $scheme - the URL scheme.
         *
         * @return string
         */
        public static function networkSiteUrl( $url, $path, $scheme ) {

            return self::filterWpLoginPhp( $url, $scheme );
        }

        /**
         * Check the current url and changes it if necessary
         *
         * @since 1.0.0
         *
         * @param string $url - the url of the site.
         * @param mixed  $scheme - the URL scheme.
         *
         * @return string
         */
        public static function filterWpLoginPhp( $url, $scheme = null ): string {

            if ( false !== strpos( $url, 'wp-login.php?action=postpass' ) ) {
                return $url;
            }

            if ( false !== strpos( $url, 'wp-login.php' ) && false === strpos( (string) \wp_get_referer(), 'wp-login.php' ) ) {

                if ( \is_ssl() ) {

                    $scheme = 'https';

                }

                $args = explode( '?', $url );

                if ( isset( $args[1] ) ) {

                    parse_str( $args[1], $args );

                    if ( isset( $args['login'] ) ) {
                        $args['login'] = rawurlencode( $args['login'] );
                    }

                    $url = \add_query_arg( $args, self::newLoginUrl( $scheme ) );

                } else {

                    $url = self::newLoginUrl( $scheme );

                }
            }

            return $url;

        }

        /**
         * Changes the login link in the welcome register email to user
         *
         * @since 1.0.0
         *
         * @param string $value - with what to replace the original login URL.
         *
         * @return string
         */
        public static function welcomeEmail( $value ): string {

            return $value = str_replace(
                'wp-login.php',
                \trailingslashit(
                    WPHelper::getOption(
                        WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_NAME,
                        WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_DEFAULT
                    )
                ),
                $value
            );

        }

        /**
         * Update url redirect : wp-admin/options.php
         *
         * @since 1.0.0
         *
         * @param mixed $loginUrl - the login url of the site.
         * @param mixed $redirect - to where to redirect.
         * @param mixed $forceReauth - should it be enforced?.
         *
         * @return string
         */
        public static function loginUrl( $loginUrl, $redirect, $forceReauth ) {
            if ( \is_404() ) {
                return '#';
            }

            if ( false === $forceReauth ) {
                return $loginUrl;
            }

            if ( empty( $redirect ) ) {
                return $loginUrl;
            }

            $redirect = explode( '?', $redirect );

            if ( \admin_url( 'options.php' ) === $redirect[0] ) {
                $loginUrl = \admin_url();
            }

            return $loginUrl;
        }

        /**
         * Sets the module name
         *
         * @param string $moduleName - module name.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function setModuleName( string $moduleName = '' ) {
            static::$moduleName = ( ( ! empty( $moduleName ) ) ? $moduleName : __( 'Login', 'secured-wp' ) );
        }

        /**
         * Extracts and extends the module info
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function getInfo(): array {

            return array_merge(
                parent::getInfo(),
                [
                    'additional_info' => [
                        'login_slug'    => [
                            'translate' => __( 'New login slug', 'secured-wp' ),
                            'value'     => self::newLoginSlug(),
                        ],
                        'redirect_slug' => [
                            'translate' => __( 'Redirect original login', 'secured-wp' ),
                            'value'     => self::newRedirectSlug(),
                        ],
                    ],
                ]
            );
        }

        /**
         * Deletes the module data
         *
         * @return boolean
         *
         * @since 1.0.0
         */
        public static function deleteModuleData(): bool {

            try {
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_NAME );
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_NEW_LOGIN_REDIRECT_VAR_NAME );
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_LOGIN_MODULE_DISABLED );
            } catch ( \Exception $e ) {
                return false;
            }

            return true;
        }

        /**
         * Resets module to its defaults
         *
         * @param string $blogId - the WP blog Id.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function restoreDefaults( $blogId = '' ) {
            self::deleteModuleData();

            self::updateNewLoginSlug( WPSEC_PLUGIN_SECURED_NEW_LOGIN_REDIRECT_VAR_DEFAULT, $blogId );
            self::updateNewRedirectSlug( WPSEC_PLUGIN_SECURED_ORIGIN_LOGIN_REDIRECT_VAR_DEFAULT, $blogId );

            self::saveGlobalEnabled( false, $blogId );
        }

        /**
         * Short module description
         *
         * @inheritDoc
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getShortDescription(): string {
            return \esc_html__( 'Hides the standard wp-login.php with your personal slug of choice. Redirects the original login to slug of your choice', 'secured-wp' );
        }

        /**
         * Returns the wpLogin status
         *
         * @since 1.0.0
         *
         * @return boolean
         */
        private static function getWpLogin(): bool {
            return self::$wpLogin;
        }

        /**
         * Sets the wpLogin status
         *
         * @since 1.0.0
         *
         * @param boolean $status - sets the module status.
         *
         * @return boolean
         */
        private static function setWpLogin( bool $status ): bool {
            self::$wpLogin = $status;
            return self::$wpLogin;
        }
    }
}
