<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers\Modules;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use WPSEC\Controllers\User;
use WPSEC\Helpers\WPHelper;
use WPSEC\Validators\Validator;

if ( ! class_exists( 'WPSEC\Controllers\Modules\LoginAttempts' ) ) {
    /**
     * Responsible for the login attempts count, store and delete
     *
     * @since 1.0.0
     */
    class LoginAttempts extends BaseModule {

        /**
         * Global setting name - stored the global value for enable / disable module
         *
         * @var string
         */
        protected static $globalSettingName = WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_MODULE_DISABLED;

        /**
         * Is the module enabled or not
         *
         * @var bool
         *
         * @since 1.0.0
         */
        protected static $moduleEnabled;

        /**
         * Holds the name of the give module
         *
         * @since 1.0.0
         *
         * @var string
         */
        protected static $moduleName;

        /**
         * User meta key name
         *
         * @since 1.0.0
         */
        const META_KEY = '_wps_login_attempts';

        /**
         * How many times the user is allowed to try to login before sending email to admin
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $allowedAttempts = null;

        /**
         * In how many minutes the user is allowed to try to login again
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $allowedMins = null;

        /**
         * Increasing login attempts for User
         *
         * @since 1.0.0
         *
         * @param \WP_User $user - the WP User.
         *
         * @return void
         */
        public static function increaseLoginAttempts( \WP_User $user ): void {
            $attempts = User::getMeta( self::META_KEY, $user, true );
            if ( '' === $attempts ) {
                $attempts = 0;
            }
            User::updateMeta( self::META_KEY, ++$attempts, $user );
        }

        /**
         * Returns the number of unsuccessful attempts for the User
         *
         * @since 1.0.0
         *
         * @param \WP_User $user - the WP User.
         *
         * @return integer
         */
        public static function getLoginAttempts( \WP_User $user ): int {
            return (int) User::getMeta( self::META_KEY, $user, true );
        }

        /**
         * Clearing login attempts for User
         *
         * @since 1.0.0
         *
         * @param \WP_User $user - the WP User.
         *
         * @return void
         */
        public static function clearLoginAttempts( \WP_User $user ): void {
            User::deleteMeta( self::META_KEY, $user );
        }

        /**
         * Returns the number of the allowed attempts
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - the WP blog id.
         *
         * @return mixed
         */
        public static function getAllowedAttempts( $blogId = '' ) {
            if ( null === self::$allowedAttempts ) {
                self::$allowedAttempts =
                WPHelper::getOption(
                    WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_VAR_NAME,
                    WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_VAR_DEFAULT,
                    '',
                    $blogId
                );
            }

            return self::$allowedAttempts;
        }

        /**
         * Deletes the stored login attempts
         *
         * @since 1.0.0
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function deleteAllowedAttempts( $blogId = '' ) {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_VAR_NAME, $blogId );
        }

        /**
         * Sets the new value for the number of allowed attempts
         *
         * @since 1.0.0
         *
         * @param mixed  $value - value which must be stored for the give blog id.
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function setAllowedAttempts( $value, $blogId = '' ) {
            if ( Validator::filterValidate( $value, 'int' ) ) {
                WPHelper::setOption( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_VAR_NAME, $value, '', $blogId );
            }
        }

        /**
         * Returns the number of the allowed attempts
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - the WP blog id.
         *
         * @return int
         */
        public static function getLockTimeMins( $blogId = '' ): int {
            if ( null === self::$allowedMins ) {
                self::$allowedMins =
                WPHelper::getOption(
                    WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_TIME_VAR_NAME,
                    WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_TIME_VAR_DEFAULT,
                    '',
                    $blogId
                );
            }

            return (int) self::$allowedMins;
        }

        /**
         * Deletes the stored login attempts
         *
         * @since 1.0.0
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function deleteLockTimeMins( $blogId = '' ) {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_TIME_VAR_NAME, $blogId );
        }

        /**
         * Sets the new value for the number of allowed attempts
         *
         * @since 1.0.0
         *
         * @param mixed  $value - the value which must be stored for the given blog.
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function setLockTimeMins( $value, $blogId = '' ) {
            if ( Validator::filterValidate( $value, 'int' ) ) {
                WPHelper::setOption( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_TIME_VAR_NAME, $value, '', $blogId );
            }
        }

        /**
         * Sets module name
         *
         * @param string $moduleName - Desired module name.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function setModuleName( string $moduleName = '' ) {
            static::$moduleName = ( ( ! empty( $moduleName ) ) ? $moduleName : __( 'Login attempts', 'secured-wp' ) );
        }

        /**
         * Extracts and extends the module info
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function getInfo(): array {

            return array_merge(
                parent::getInfo(),
                [
                    'additional_info' => [
                        'login_attempts' => [
                            'translate' => __( 'Login Attempts', 'secured-wp' ),
                            'value'     => self::getAllowedAttempts(),
                        ],
                        'lock_time'      => [
                            'translate' => __( 'Lock time', 'secured-wp' ),
                            'value'     => self::getLockTimeMins() . __( ' mins.', 'secured-wp' ),
                        ],
                    ],
                ]
            );
        }

        /**
         * Deletes the module data
         *
         * @return boolean
         *
         * @since 1.0.0
         */
        public static function deleteModuleData(): bool {

            try {
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_VAR_NAME );
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_TIME_VAR_NAME );
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_MODULE_DISABLED );
            } catch ( \Exception $e ) {
                return false;
            }

            return true;
        }

        /**
         * Short module description
         *
         * @inheritDoc
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getShortDescription(): string {
            return \esc_html__( 'Counts incorect login attempts and locks the user - use standard user menu to check locked users.', 'secured-wp' );
        }

        /**
         * Resets module to its defaults
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function restoreDefaults( $blogId = '' ) {
            self::deleteModuleData();

            self::setAllowedAttempts( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_VAR_DEFAULT, $blogId );
            self::setLockTimeMins( WPSEC_PLUGIN_SECURED_LOGIN_ATTEMPTS_TIME_VAR_DEFAULT, $blogId );

            self::saveGlobalEnabled( false, $blogId );
        }
    }
}
