<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers\Modules;

use OTPHP\TOTP;
use Mobile_Detect;
use WPSEC\Helpers\WPHelper;
use WPSEC\Validators\Validator;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Controllers\Modules\RememberMe' ) ) {

    /**
     * Sets remember me cookie and sets the authentication in the system
     *
     * @since 1.0.0
     */
    class RememberMe extends BaseModule {

        /**
         * Global setting name - stored the global value for enable / disable module
         *
         * @var string
         */
        protected static $globalSettingName = WPSEC_PLUGIN_SECURED_REMEMBER_MODULE_DISABLED;

        /**
         * Keeps the name of the transient for the remember me, and also the name used for the cookie name
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $transientPrefix = '_remember_me_';

        /**
         * Sets the time period for how long the TOTP is valid
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $period = null;

        /**
         * How many digits the password must be
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $digits = 12;

        /**
         * Which algorithm is used for generating the proper TOTP
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $algorithm = 'sha1';

        /**
         * In how many days the user is allowed to use remember me
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $allowedDays = null;

        /**
         * Holds the name of the give module
         *
         * @since 1.0.0
         *
         * @var string
         */
        protected static $moduleName;

        /**
         * Sets the remember me cookie and sets the remember me transient
         *
         * @since 1.0.0
         *
         * @param mixed $userId - WP user ID.
         *
         * @return void
         */
        public static function setRememberMe( $userId ) {
            $secret = self::getSecret();
            self::setCookie( $userId, $secret['pass'] );
            self::setTransient( $userId, $secret['secret'] );
        }

        /**
         * Removes the cookie and transient if they are present
         *
         * @since 1.0.0
         *
         * @param mixed $userId - WP user ID.
         *
         * @return void
         *
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function clearRememberMe( $userId ) {
            if ( isset( $_COOKIE[ self::$transientPrefix ] ) ) {
                self::removeTransient( $userId );
                self::deleteRememberMeCookie();
            }
        }

        /**
         * Checks the cookie value against the stored value in the transient
         * Device is also checked
         *
         * @since 1.0.0
         *
         * @param mixed $userId - WP user ID.
         *
         * @return bool|int - false if all the checks fail, user id otherwise
         *
         * @SuppressWarnings(PHPMD.Superglobals)
         * @SuppressWarnings(PHPMD.ExitExpression)
         */
        public static function checkRememberMe( $userId ) {
            if ( $userId ) {
                return $userId;
            }

            $rememberMe = isset( $_COOKIE[ self::$transientPrefix ] ) ? \sanitize_text_field( \wp_unslash( $_COOKIE[ self::$transientPrefix ] ) ) : false;

            try {
                if ( $rememberMe ) {

                    $vals = \json_decode( \stripslashes_deep( $rememberMe ), true, JSON_THROW_ON_ERROR );

                    $device = self::getDevice();

                    /**
                     * Collect data from the transient
                     */
                    $rememberTransient = \get_transient( self::$transientPrefix . md5( $device ) . '_' . $vals['uid'] );

                    if ( $rememberTransient ) {
                        /**
                         * Builds the TOTP class
                         */
                        $otp = TOTP::create(
                            $rememberTransient['secret'],
                            self::getPeriod(),
                            self::$algorithm,
                            self::$digits
                        );

                        /**
                         * Check password against the TOTP object
                         */
                        if ( $vals['pass'] === $otp->at( time() ) ) {
                            \wp_set_current_user( $vals['uid'] );
                            \wp_set_auth_cookie( $vals['uid'] );

                            \wp_safe_redirect( ( ( isset( $_SERVER['HTTPS'] ) && 'off' !== $_SERVER['HTTPS'] ) ? 'https' : 'http' ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]" ); // @codingStandardsIgnoreLine

                            exit();
                        }
                    } else {
                        return false;
                    }
                }
            } catch ( \Exception $exc ) {
                return false;
            }

            return false;
        }

        /**
         * Returns the name of the transient prefix
         *
         * @since 1.0.0
         *
         * @return string
         */
        public static function getTransientPrefix(): string {
            return self::$transientPrefix;
        }

        /**
         * Returns the device of the current user based on the browser headers
         *
         * @since 1.0.0
         *
         * @return string
         */
        public static function getDevice(): string {
            $deviceDetect = new Mobile_Detect();
            $device       = $deviceDetect->getUserAgent();
            if ( null === $device ) {
                $device = '';
            }

            return $device;
        }

        /**
         * Returns the expiry time of the remember me transient, based on the given device
         *
         * @since 1.0.0
         *
         * @param string  $device - device (string).
         * @param integer $userId - the WP user.
         *
         * @return integer
         */
        public static function getExpireTime( string $device, int $userId ): int {
            $expires = (int) WPHelper::getOption( '_transient_timeout_' . self::$transientPrefix . md5( $device ) . '_' . $userId, 0 );

            return $expires;
        }

        /**
         * Removes remember me transient
         *
         * @since 1.0.0
         *
         * @param [type] $userId - the WP user.
         * @param mixed  $device - device (string).
         *
         * @return void
         */
        public static function removeTransient( $userId, $device = null ) {
            if ( null === $device ) {
                $device = self::getDevice();
            }

            \delete_transient(
                self::$transientPrefix . md5( $device ) . '_' . $userId,
            );
        }

        /**
         * Returns the number of the allowed attempts
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - the WP blog id.
         *
         * @return int
         */
        public static function getLockTimeDays( $blogId = '' ): int {
            if ( null === self::$allowedDays ) {
                self::$allowedDays =
                WPHelper::getOption(
                    WPSEC_PLUGIN_SECURED_REMEMBER_TIME_VAR_NAME,
                    WPSEC_PLUGIN_SECURED_REMEMBER_TIME_VAR_DEFAULT,
                    '',
                    $blogId
                );
            }

            return (int) self::$allowedDays;
        }

        /**
         * Deletes the stored days for remember me
         *
         * @since 1.0.0
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function deleteLockTimeMins( $blogId = '' ) {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_REMEMBER_TIME_VAR_NAME, $blogId );
        }

        /**
         * Sets the new value for the number of allowed days to use remember device
         *
         * @since 1.0.0
         *
         * @param mixed  $value - the value which must be stored for the given blog.
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function setLockTimeDays( $value, $blogId = '' ) {
            if ( Validator::filterValidate( $value, 'int' ) ) {
                WPHelper::setOption( WPSEC_PLUGIN_SECURED_REMEMBER_TIME_VAR_NAME, $value, '', $blogId );
            }
        }

        /**
         * Deletes the module data
         *
         * @return boolean
         *
         * @since 1.0.0
         */
        public static function deleteModuleData(): bool {

            try {
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_REMEMBER_TIME_VAR_NAME );
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_REMEMBER_MODULE_DISABLED );
            } catch ( \Exception $e ) {
                return false;
            }

            return true;
        }

        /**
         * Sets module name
         *
         * @param string $moduleName - Desired module name.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function setModuleName( string $moduleName = '' ) {
            static::$moduleName = ( ( ! empty( $moduleName ) ) ? $moduleName : __( 'Remember Devices', 'secured-wp' ) );
        }

        /**
         * Extracts and extends the module info
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function getInfo(): array {

            return array_merge(
                parent::getInfo(),
                [
                    'additional_info' => [
                        'remember_time' => [
                            'translate' => __( 'Lock time', 'secured-wp' ),
                            'value'     => self::getLockTimeDays() . __( ' days.', 'secured-wp' ),
                        ],
                    ],
                ]
            );
        }

        /**
         * Resets module to its defaults
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function restoreDefaults( $blogId = '' ) {
            self::deleteModuleData();

            self::setLockTimeDays( WPSEC_PLUGIN_SECURED_REMEMBER_TIME_VAR_DEFAULT, $blogId );

            self::saveGlobalEnabled( false, $blogId );
        }

        /**
         * Returns the current period
         * This is based on current settings - that means that this will return different values based on global settings changes
         * That is related to the expiration time as well
         *
         * TODO: Make sure that this acts properly when there are already created transients
         *
         * @return int
         *
         * @since 1.0.0
         */
        public static function getPeriod(): int {
            if ( null === self::$period ) {
                self::$period = DAY_IN_SECONDS * self::getLockTimeDays();
            }

            return (int) self::$period;
        }

        /**
         * Short module description
         *
         * @inheritDoc
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getShortDescription(): string {
            return \esc_html__( 'When enabled, if user checks remember me on login, the device will be stored and the user wont be asked for credentials for the given amount of time.', 'secured-wp' );
        }

        /**
         * Resets the remember me cookie
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function deleteRememberMeCookie() {
            $domain = ( defined( 'COOKIE_DOMAIN' ) && COOKIE_DOMAIN ) ? COOKIE_DOMAIN : '/';
            setcookie(
                self::$transientPrefix,
                '',
                [
                    'expires'  => time() + self::getPeriod(),
                    'path'     => $domain,
                    'domain'   => '',
                    'samesite' => 'Strict',
                    'secure'   => false,
                    'httponly' => true,
                ]
            );
        }

        /**
         * Sets the remember me cookie
         *
         * @since 1.0.0
         *
         * @param [type] $userId - the WP user.
         * @param [type] $pass - pass for the given user.
         *
         * @return void
         */
        private static function setCookie( $userId, $pass ) {
            $domain = ( defined( 'COOKIE_DOMAIN' ) && COOKIE_DOMAIN ) ? COOKIE_DOMAIN : '/';
            $device = self::getDevice();
            setcookie(
                self::$transientPrefix,
                \json_encode(
                    [
                        'uid'    => $userId,
                        'pass'   => $pass,
                        'device' => $device,
                    ]
                ),
                [
                    'expires'  => time() + self::getPeriod(),
                    'path'     => $domain,
                    'domain'   => '',
                    'samesite' => 'Strict',
                    'secure'   => false,
                    'httponly' => true,
                ]
            );
        }

        /**
         * Sets transient in the WordPress to check against
         *
         * @since 1.0.0
         *
         * @param [type] $userId - the WP user.
         * @param [type] $secret - secret TOTP.
         *
         * @return void
         */
        private static function setTransient( $userId, $secret ) {
            $device = self::getDevice();

            \set_transient(
                self::$transientPrefix . md5( $device ) . '_' . $userId,
                [
                    'uid'    => $userId,
                    'secret' => $secret,
                    'device' => $device,
                ],
                self::getPeriod()
            );
        }

        /**
         * Generates TOTP secrets
         *
         * @since 1.0.0
         *
         * @return array
         */
        private static function getSecret(): array {

            $otp = TOTP::create(
                null, // Let the secret be defined by the class.
                self::getPeriod(), // The period.
                self::$algorithm, // The digest algorithm.
                self::$digits // The output will generate 6 digits.
            );

            return [
                'pass'   => $otp->at( time() ),
                'secret' => $otp->getSecret(),
            ];
        }
    }
}
