<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers\Modules;

use WPSEC\Helpers\WPHelper;
use WPSEC\Validators\Validator;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Controllers\Modules\TwoFASettings' ) ) {
    /**
     * Responsible for storing and extracting the user settings for the methods
     * TOTP and OOB
     *
     * @since 1.0.0
     */
    class TwoFASettings extends BaseModule {

        /**
         * Global setting name - stored the global value for enable / disable module
         *
         * @var string
         */
        protected static $globalSettingName = WPSEC_PLUGIN_SECURED_2FA_MODULE_DISABLED;

        /**
         * Is the module enabled or not
         *
         * @var bool
         *
         * @since 1.0.0
         */
        protected static $moduleEnabled;

        /**
         * Holds the name of the give module
         *
         * @since 1.0.0
         *
         * @var string
         */
        protected static $moduleName;

        /**
         * Holds the status of the totp global setting
         *
         * @since 1.0.0
         *
         * @var bool
         */
        private static $totpEnabled = null;

        /**
         * Holds the status of the oob global setting
         *
         * @since 1.0.0
         *
         * @var bool
         */
        private static $oobEnabled = null;

        /**
         * Returns the status of totp
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - the WP blog id.
         *
         * @return mixed
         */
        public static function isTotpEnabled( $blogId = '' ) {
            if ( null === self::$totpEnabled ) {
                self::$totpEnabled =
                WPHelper::getOption(
                    WPSEC_PLUGIN_SECURED_2FA_TOTP_VAR_NAME,
                    WPSEC_PLUGIN_SECURED_2FA_TOTP_VAR_DEFAULT,
                    '',
                    $blogId
                );

                self::$totpEnabled = filter_var( self::$totpEnabled, FILTER_VALIDATE_BOOLEAN );
            }

            return self::$totpEnabled;
        }

        /**
         * Returns the status of oob
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - the WP blog id.
         *
         * @return mixed
         */
        public static function isOobEnabled( $blogId = '' ) {
            if ( null === self::$oobEnabled ) {
                self::$oobEnabled =
                WPHelper::getOption(
                    WPSEC_PLUGIN_SECURED_2FA_OOB_VAR_NAME,
                    WPSEC_PLUGIN_SECURED_2FA_OOB_VAR_DEFAULT,
                    '',
                    $blogId
                );

                self::$oobEnabled = filter_var( self::$oobEnabled, FILTER_VALIDATE_BOOLEAN );
            }

            return self::$oobEnabled;
        }

        /**
         * Deletes the stored totp status
         *
         * @since 1.0.0
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function deleteTotp( $blogId = '' ) {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_2FA_TOTP_VAR_NAME, $blogId );
        }

        /**
         * Sets the new value for oob settings
         *
         * @since 1.0.0
         *
         * @param bool   $value - the desired value which must be stored for the given blog id.
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function setTotp( $value, $blogId = '' ) {
            if ( Validator::filterValidate( $value, 'bool' ) ) {
                WPHelper::setOption( WPSEC_PLUGIN_SECURED_2FA_TOTP_VAR_NAME, $value, '', $blogId );
            }
        }

        /**
         * Deletes the stored oob status
         *
         * @since 1.0.0
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function deleteOob( $blogId = '' ) {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_2FA_OOB_VAR_NAME, $blogId );
        }

        /**
         * Sets the new value for oob settings
         *
         * @since 1.0.0
         *
         * @param bool   $value - the desired value which must be stored for the given blog id.
         * @param string $blogId - the WP blog id.
         *
         * @return void
         */
        public static function setOob( $value, $blogId = '' ) {
            if ( Validator::filterValidate( $value, 'bool' ) ) {
                WPHelper::setOption( WPSEC_PLUGIN_SECURED_2FA_OOB_VAR_NAME, $value, '', $blogId );
            }
        }

        /**
         * Sets the module name
         *
         * @param string $moduleName - Sets the module name.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function setModuleName( string $moduleName = '' ) {
            static::$moduleName = ( ( ! empty( $moduleName ) ) ? $moduleName : __( '2FA settings', 'secured-wp' ) );
        }

        /**
         * Extracts and extends the module info
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function getInfo(): array {

            return array_merge(
                parent::getInfo(),
                [
                    'additional_info' => [
                        'totp' => [
                            'translate' => __( 'TOTP enabled', 'secured-wp' ),
                            'value'     => ( ( self::isTotpEnabled() ) ? __( 'true', 'secured-wp' ) : __( 'false', 'secured-wp' ) ),
                        ],
                        'oob'  => [
                            'translate' => __( 'OOB enabled', 'secured-wp' ),
                            'value'     => ( ( self::isOobEnabled() ) ? __( 'true', 'secured-wp' ) : __( 'false', 'secured-wp' ) ),
                        ],
                    ],
                ]
            );
        }

        /**
         * Deletes the module data
         *
         * @return boolean
         *
         * @since 1.0.0
         */
        public static function deleteModuleData(): bool {

            try {
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_2FA_TOTP_VAR_NAME );
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_2FA_OOB_VAR_NAME );
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_2FA_MODULE_DISABLED );
            } catch ( \Exception $e ) {
                return false;
            }

            return true;
        }

        /**
         * Resets module to its defaults
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function restoreDefaults( $blogId = '' ) {
            self::deleteModuleData();

            self::setTotp( WPSEC_PLUGIN_SECURED_2FA_TOTP_VAR_DEFAULT, $blogId );
            self::setOob( WPSEC_PLUGIN_SECURED_2FA_OOB_VAR_DEFAULT, $blogId );

            self::saveGlobalEnabled( false, $blogId );
        }

        /**
         * Short module description
         *
         * @inheritDoc
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getShortDescription(): string {
            return \esc_html__( 'Enables 2FA. Next time, when user logins s/he will be asked to add the site to Authenticator application of their choice, and from now on they must provide code from that App in order to login.', 'secured-wp' );
        }
    }
}
