<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers\Modules;

use WPSEC\Helpers\WPHelper;
use WPSEC\Validators\Validator;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Controllers\Modules\XMLRPCprevents' ) ) {
    /**
     * Responsible for preventing the XML-RPC for the given WP installation
     *
     * @since 1.0.0
     */
    class XMLRPCprevents extends BaseModule {

        /**
         * Global setting name - stored the global value for enable / disable module
         *
         * @var string
         */
        protected static $globalSettingName = WPSEC_PLUGIN_SECURED_XMLRPC_MODULE_DISABLED;

        /**
         * Is the module enabled or not
         *
         * @var bool
         *
         * @since 1.0.0
         */
        protected static $moduleEnabled;

        /**
         * Holds the name of the give module
         *
         * @since 1.0.0
         *
         * @var string
         */
        protected static $moduleName;

        /**
         * Sets the module name
         *
         * @param string $moduleName - Sets the module name.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function setModuleName( string $moduleName = '' ) {
            static::$moduleName = ( ( ! empty( $moduleName ) ) ? $moduleName : __( 'XML-RPC settings', 'secured-wp' ) );
        }

        /**
         * Deletes the module data
         *
         * @return boolean
         *
         * @since 1.0.0
         */
        public static function deleteModuleData(): bool {

            try {
                WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_XMLRPC_MODULE_DISABLED );
            } catch ( \Exception $e ) {
                return false;
            }

            return true;
        }

        /**
         * Resets module to its defaults
         *
         * @param string $blogId - the WP blog id.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function restoreDefaults( $blogId = '' ) {
            self::deleteModuleData();

            self::saveGlobalEnabled( false, $blogId );
        }

        /**
         * Short module description
         *
         * @inheritDoc
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getShortDescription(): string {
            return \esc_html__( 'Disables XML-RPC. By default WordPress has that enabled - this is potentially security risk - that will disable XML-RPC on your site completely', 'secured-wp' );
        }

        /**
         * Removes all the XML-RPC methods
         *
         * @param array $methods - array with all the XML-RPC methods.
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function removeXmlrpcMethods( $methods ): array {
            $methods = []; // empty the array.
            return $methods;
        }

        /**
         * Removes all the XML-RPC link
         *
         * @param array $headers - array with headers.
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function removeXmlrpcLink( $headers ): array {
            unset( $headers['X-Pingback'] );

            return $headers;
        }

        /**
         * Strips out ping-back link from html head
         *
         * @param [type] $output - Output collected for sending.
         * @param [type] $property - Currently parsed property.
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function removeHeaderPingLink( $output, $property ) {
            return ( 'pingback_url' === $property ) ? null : $output;
        }
    }
}
