<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Controllers;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use OTPHP\TOTP;
use WPSEC\Helpers\WPHelper;
use WPSEC\Validators\Validator;
use WPSEC\Helpers\SecretsGenerator;
use WPSEC\Controllers\Modules\LoginAttempts;
use WPSEC\Controllers\Modules\RememberMe;

if ( ! class_exists( 'WPSEC\Controllers\User' ) ) {
    /**
     * Holds information about the user
     *
     * @since 1.0.0
     */
    class User {

        /**
         * Holds the current user
         *
         * @since 1.0.0
         *
         * @var mixed
         */
        private static $user = null;

        /**
         * Prefix of the used transient for the user locks
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $transientPrefix = 'attempted_login_';

        /**
         * Prefix of the used transient for the user logged in devices
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $transientPrefixLogged = '_logged_in_devices';

        /**
         * Totpkey of the user
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $totpKey = null;

        /**
         * Holds the name of the metakey for the user totp
         *
         * @var string
         *
         * @since 1.0.0
         */
        private static $totpKeyMetaKeyName = '_wpsec_totp_key';

        /**
         * Holds the status name for the TOTP of the user
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $totEnabledKey = '_wpsec-totp-enabled';

        /**
         * Holds the status name for the TOTP of the user
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $userExcludedTwoFaKey = '_wpsec-two-fa-excluded';

        /**
         * Sets the time period for how long the TOTP is valid
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $period = 30;

        /**
         * How many digits the password must be
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $digits = 6;

        /**
         * Which algorithm is used for generating the proper TOTP
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $algorithm = 'sha1';

        /**
         * Holds the current status of the user - logged or not
         *
         * @since 1.0.0
         *
         * @var bool
         */
        private static $loggedIn = null;

        /**
         * Returns the current user
         * TODO: re-check that logic - there is maybe room for some improvements
         *
         * @param mixed $user - User - that could be int \WP_User or null, if nothing is provided the logic will try to extract user from the WP - currently logged in.
         *
         * @since 1.0.0
         *
         * @return \WP_User
         */
        public static function getUser( $user = null ): \WP_User {
            if ( null === self::$user || ! is_null( $user ) ) {
                self::setUser( $user );
            }

            return self::$user;
        }

        /**
         * Sets the current user.
         *
         * @since 1.0.0
         *
         * @param mixed $user user object @see \WP_user or integer representing the ID of the user. If null / not provided, tries to extract the current user.
         *
         * @return WPSEC\Controllers\User
         *
         * @throws \Exception - throws exception if can not extract user.
         */
        public static function setUser( $user = null ) {
            self::$user = null;

            if ( empty( $user ) ) {
                self::$user = \wp_get_current_user();
            }

            if ( is_null( self::$user ) && Validator::filterValidate( $user, 'int' ) ) {
                self::$user = \get_user_by( 'id', $user );
            }

            if ( is_null( self::$user ) && $user instanceof \WP_User ) {
                self::$user = $user;
            }

            if ( is_null( self::$user ) ) {
                self::$user = \get_user_by( 'login', $user );
                if ( false === self::$user ) {
                    self::$user = \get_user_by( 'email', $user );
                }
            }

            if ( false === self::$user && Validator::filterValidate( $user, 'int' ) ) {
                self::$user = new \WP_User( $user );
            }

            if ( is_null( self::$user ) ) {
                throw new \Exception( 'Can not extract user', 1 );
            }

            return __CLASS__;
        }

        /**
         * Gets currently stored meta for given user
         *
         * @since 1.0.0
         *
         * @param string  $metaKey - the meta key which has to be checked for value.
         * @param mixed   $user - could be User instance, user Id or null - if null currently logged user is used.
         * @param boolean $single - return single or array.
         *
         * @return mixed
         */
        public static function getMeta( string $metaKey, $user = null, $single = false ) {
            return \get_user_meta( self::getUser( $user )->ID, $metaKey, $single );
        }

        /**
         * Updates given user meta with given value
         *
         * @since 1.0.0
         *
         * @param string $metaKey - the meta key which has to be checked for value.
         * @param mixed  $metaValue - value for the meta which needs to be updated.
         * @param mixed  $user - could be User instance, user Id or null - if null currently logged user is used.
         *
         * @return void
         */
        public static function updateMeta( string $metaKey, $metaValue, $user = null ) {
            \update_user_meta( self::getUser( $user )->ID, $metaKey, $metaValue );
        }

        /**
         * That method is just an alias for @see updateMeta
         *
         * @since 1.0.0
         *
         * @param string $metaKey - key which identifies the meta.
         * @param mixed  $metaValue - value which must be stored.
         * @param mixed  $user - the user which that meta applies to.
         * @return void
         */
        public static function setMeta( string $metaKey, $metaValue, $user = null ) {
            self::updateMeta( $metaKey, $metaValue, $user );
        }

        /**
         * Deletes meta for the user
         *
         * @since 1.0.0
         *
         * @param string $metaKey - the meta key which has to be checked for value.
         * @param mixed  $user - could be User instance, user Id or null - if null currently logged user is used.
         *
         * @return void
         */
        public static function deleteMeta( string $metaKey, $user = null ) {
            \delete_user_meta( self::getUser( $user )->ID, $metaKey );
        }

        /**
         * Checks if the given user is locked
         *
         * @since 1.0.0
         *
         * @param mixed $user - @see setUser method of this class.
         *
         * @return boolean
         */
        public static function isLocked( $user = null ): bool {
            self::setUser( $user );

            return false !== \get_transient( self::$transientPrefix . self::$user->ID ) ? true : false;
        }

        /**
         * Gets the locked user transient name
         *
         * @since 1.0.0
         *
         * @return string
         */
        public static function getLockedTransientPrefix(): string {
            return self::$transientPrefix;
        }

        /**
         * Lock user and destroys its session
         *
         * @since 1.0.0
         *
         * @param mixed $user - @see setUser method of this class.
         * @param bool  $destroySessions - should we also destroy all the sessions for the user?.
         *
         * @return void
         */
        public static function lockUser( $user = null, $destroySessions = false ): void {
            self::setUser( $user );

            \set_transient(
                self::getLockedTransientPrefix() . self::$user->ID,
                self::$user->user_login,
                60 * LoginAttempts::getLockTimeMins()
            );

            if ( $destroySessions ) {
                $manager = \WP_Session_Tokens::get_instance( self::$user->ID );
                $manager->destroy_all();
            }
        }

        /**
         * Unlocks the user and clears the login attempts
         *
         * @since 1.0.0
         *
         * @param mixed $user - @see setUser method of this class.
         *
         * @return void
         */
        public static function unlockUser( $user = null ): void {
            self::setUser( $user );

            \delete_transient(
                self::getLockedTransientPrefix() . self::$user->ID
            );

            LoginAttempts::clearLoginAttempts( $user );
        }

        /**
         * Returns the user secret key, if one is not exist - generates it
         *
         * @since 1.0.0
         *
         * @param mixed $user - user which TOTP key is needed.
         *
         * @return string
         */
        public static function getUserTotp( $user = null ) {
            if ( null === self::$totpKey ) {
                if ( '' === self::$totpKey = self::getMeta( self::$totpKeyMetaKeyName, $user, true ) ) {

                    self::$totpKey = TOTP::create(
                        SecretsGenerator::totpGenerateKey(),
                        self::$period,
                        self::$algorithm,
                        self::$digits
                    )->getSecret();
                    self::updateMeta( self::$totpKeyMetaKeyName, self::$totpKey, $user );
                }
            }

            return self::$totpKey;
        }

        /**
         * Deletes user meta
         *
         * @param mixed $user - The user for which the TOTP key must be deleted.
         *
         * @return void
         *
         * @since 1.3
         */
        public static function deleteUserTotp( $user = null ) {
            self::deleteMeta( self::$totpKeyMetaKeyName, $user );
            self::removeUserTotpEnabledMeta( $user );
        }

        /**
         * Validates authentication.
         *
         * @since 1.0.0
         *
         * @param string           $authCode - Authentication code.
         * @param null|int|WP_User $user WP_User object of the logged-in user.
         *
         * @return bool Whether the user gave a valid code
         */
        public static function validateTotpAuthentication( string $authCode, $user = null ) {

            $authCode = str_replace( [ ' ' ], '', $authCode );

            $totp = TOTP::create(
                self::getUserTotp( $user ),
                self::$period,
                self::$algorithm,
                self::$digits
            );

            return $totp->verify( \sanitize_text_field( $authCode ) );
        }

        /**
         * Returns the totp status of the current user
         *
         * @since 1.0.0
         *
         * @param null|int|WP_User $user WP_User object of the logged-in user.
         *
         * @return boolean
         */
        public static function isTotpUserEnabled( $user = null ): bool {
            return \filter_var( self::getMeta( self::$totEnabledKey, $user, true ), FILTER_VALIDATE_BOOLEAN );
        }

        /**
         * Sets the TOTP as enabled for the user
         *
         * @param null|int|WP_User $user WP_User object of the logged-in user.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function enableTotp( $user = null ) {
            self::setMeta( self::$totEnabledKey, true, $user );
        }

        /**
         * Remove the TOTP meta key completely
         *
         * @param mixed $user - The user for which the meta must be removed.
         *
         * @return void
         *
         * @since 1.3
         */
        public static function removeUserTotpEnabledMeta( $user = null ) {
            self::deleteMeta( self::$totEnabledKey, $user );
        }

        /**
         * Returns the 2FA status of the current user
         *
         * @since 1.0.0
         *
         * @param null|int|WP_User $user WP_User object of the logged-in user.
         *
         * @return boolean
         */
        public static function isTwoFaUserExcluded( $user = null ): bool {
            return filter_var( self::getMeta( self::$userExcludedTwoFaKey, $user, true ), FILTER_VALIDATE_BOOLEAN );
        }

        /**
         * Sets the 2FA status as excluded for the user
         *
         * @param null|int|WP_User $user WP_User object of the logged-in user.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function excludeTwoFa( $user = null ) {
            self::setMeta( self::$userExcludedTwoFaKey, true, $user );
        }

        /**
         * Sets the 2FA status as excluded for the user
         *
         * @param null|int|WP_User $user WP_User object of the logged-in user.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function includeTwoFa( $user = null ) {
            self::deleteMeta( self::$userExcludedTwoFaKey, $user );
        }

        /**
         * Sets the logged in devices in the user's array
         *
         * @since 1.0.0
         *
         * @param string $device - the device name - check @RememberMe class for more information.
         * @param mixed  $user - The user for which device is se.
         *
         * @return void
         */
        public static function setLoggedInDevice( string $device, $user = null ) {
            $devices = self::getLoggedInDevices( $user );

            if ( ! empty( $devices ) && is_array( $devices ) ) {
                if ( ! in_array( $device, $devices, true ) ) {
                    $devices[] = $device;
                }
            } else {
                $devices = [ $device ];
            }

            self::setMeta( self::$transientPrefixLogged, $devices, $user, true );
        }

        /**
         * Deletes given device from the remember me array of the user,
         *
         * @since 1.0.0
         *
         * @param string $device - the device name - check @RememberMe class for more information.
         * @param mixed  $user - could the int \WP_User or null - class will try to extract the proper user.
         *
         * @return bool
         */
        public static function deleteLoggedInDevice( string $device, $user = null ): bool {
            $devices = self::getLoggedInDevices( $user );
            if ( ! empty( $devices ) && is_array( $devices ) ) {
                if ( ! in_array( $device, $devices, true ) ) {
                    return false;
                }
                $key = array_search( $device, $devices, true );
                unset( $devices[ $key ] );
                self::setMeta( self::$transientPrefixLogged, $devices, $user, true );

                RememberMe::removeTransient( self::$user->ID, $device );

                return true;
            }

            return false;
        }

        /**
         * Removes all the logged in devices for the given user, and removes the transients
         *
         * @since 1.0.0
         *
         * @param mixed $user - could the int \WP_User or null - class will try to extract the proper user.
         *
         * @return boolean
         */
        public static function deleteAllLoggedInDevices( $user = null ): bool {
            $devices = self::getLoggedInDevices( $user );
            if ( ! empty( $devices ) && is_array( $devices ) ) {
                foreach ( $devices as $device ) {
                    RememberMe::removeTransient( self::$user->ID, $device );
                }
            }

            self::setMeta( self::$transientPrefixLogged, [], $user, true );

            return true;
        }

        /**
         * Cleans up the data of the logged in devices for the user based on the expired transients for the every device
         * TODO: maybe it is better that to be part of the RememberMe class
         *
         * @since 1.0.0
         *
         * @param mixed $user - could the int \WP_User or null - class will try to extract the proper user.
         *
         * @return void
         */
        public static function cleanLoggedInDevices( $user = null ) {
            $devices        = $originalDevices = self::getLoggedInDevices( $user );
            $rememberPrefix = RememberMe::getTransientPrefix();

            foreach ( $devices as $deviceId => &$loggedDevice ) {

                $dataTimeout = WPHelper::getOption(
                    '_transient_timeout_' . $rememberPrefix . md5( $loggedDevice ) . '_' . self::getUser( $user )->ID
                );

                if ( ! $dataTimeout || ! ( $dataTimeout > time() ) ) {
                    unset( $devices[ $deviceId ] );
                }
            }
            unset( $loggedDevice );

            if ( $devices !== $originalDevices ) {
                self::setMeta( self::$transientPrefixLogged, $devices, $user, true );
            }
        }

        /**
         * Returns the stored logged in devices for the given user
         *
         * @since 1.0.0
         *
         * @param mixed $user - could the int \WP_User or null - class will try to extract the proper user.
         *
         * @return array - all the stored devices or false if there aren't any
         */
        public static function getLoggedInDevices( $user = null ): array {
            $devices = self::getMeta( self::$transientPrefixLogged, $user, true );

            if ( ! is_array( $devices ) ) {
                $devices = [];
            }
            return $devices;
        }

        /**
         * Checks if any given user is logged in
         *
         * That method is used when batch jobs is in place - like the user list in the user menu.
         * That method must not be used if you want to determine the status of the currently logged user,
         * but just to check the status of the any given user, as it will search the entire user meta table
         * in order to return correct results.
         *
         * If you want to get the status of the current user - @see isCurrentlyLogged method of this class
         *
         * @since 1.0.0
         *
         * @param mixed $user - could the int \WP_User or null - class will try to extract the proper user.
         *
         * @return boolean
         */
        public static function isLogged( $user = null ): bool {
            self::setUser( $user );

            $loggedUsers = Users::getLoggedUsers();

            $ids = \wp_list_pluck( $loggedUsers, 'ID' );

            return in_array( self::$user->ID, $ids, true );
        }

        /**
         * Returns HTML statistic for the current user
         *
         * @param mixed $user - the user.
         *
         * @return string
         *
         * @since
         */
        public static function getStatus( $user = null ): string {
            $status = [
                '2FAEnabled'  => __( '2FA enabled: YES', 'secured-wp' ),
                'TotpEnabled' => __( 'TOTP enabled: NO', 'secured-wp' ),
            ];

            if ( self::isTwoFaUserExcluded( $user ) ) {
                $status['2FAEnabled'] = __( '2FA enabled: EXCLUDED', 'secured-wp' );
            }

            if ( self::isTotpUserEnabled( $user ) ) {
                $status['TotpEnabled'] = __( 'TOTP enabled: YES', 'secured-wp' );
            }

            return implode( '<br/>', $status );
        }

        /**
         * Returns the status of the current user - is it logged or not
         *
         * Unlike @see isLogged method of this class - it is checking only the current user
         *
         * If the user is logged in, this method also set the $user property of this class for caching purposes
         *
         * @return boolean
         *
         * @since 1.0.0
         */
        public static function isCurrentlyLogged() {
            if ( null === self::$loggedIn ) {
                self::$loggedIn = false;
                if ( \is_user_logged_in() ) {
                    self::$loggedIn = true;
                    self::setUser();
                }
            }

            return self::$loggedIn;
        }
    }
}
