<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Helpers;

use WPSEC\Controllers\User;
use WPSEC\Validators\Validator;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Helpers\AjaxRequests' ) ) {
    /**
     * AJAX requests helper class
     *
     * @since 1.0.0
     */
    class AjaxRequests {

        /**
         * Initiates and sets the AJAX methods used from the plugin in the admin section of the WP
         *
         * @return void
         */
        public static function initAdmin() {

            // delete logged in device for user.
            \add_action( 'wp_ajax_nopriv_logged_device_delete', [ __CLASS__, 'deleteRememberMeDevice' ] );
            \add_action( 'wp_ajax_logged_device_delete', [ __CLASS__, 'deleteRememberMeDevice' ] );

            // delete all logged in devices for user.
            \add_action( 'wp_ajax_nopriv_all_logged_device_delete', [ __CLASS__, 'deleteAllRememberMeDevices' ] );
            \add_action( 'wp_ajax_all_logged_device_delete', [ __CLASS__, 'deleteAllRememberMeDevices' ] );

            // delete all logged in devices for user.
            \add_action( 'wp_ajax_nopriv_wps_delete_qr', [ __CLASS__, 'deleteQRCodeForUser' ] );
            \add_action( 'wp_ajax_wps_delete_qr', [ __CLASS__, 'deleteQRCodeForUser' ] );
        }

        /**
         * AJAX request for deleting the device from the logged in devices for given user
         * It also can be used outside AJAX requests
         *
         * @since 1.0.0
         *
         * ! WP do_action sends first empty parameter if there are no parameters
         * @param string $device - if that is set to null, global POST array will be checked.
         * @param mixed  $user - WP user for which remember me must be deleted.
         *
         * @return void
         *
         * @SuppressWarnings(PHPMD.ExitExpressions)
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function deleteRememberMeDevice( string $device = null, $user = null ) {
            if ( ! isset( $_POST['nonce'] ) ||
                empty( $_POST['nonce'] ) ||
                ! \wp_verify_nonce( \sanitize_text_field( \wp_unslash( $_POST['nonce'] ) ), 'wp-secured-delete_device-ajax-nonce' ) ) {
                die();
            }

            if ( empty( $device ) && isset( $_POST['device'] ) && ! empty( $_POST['device'] ) ) {
                $device = base64_decode( \sanitize_text_field( \wp_unslash( $_POST['device'] ) ) );
            }

            if ( ! isset( $_POST['user'] ) || empty( $_POST['user'] ) ) {
                die();
            }
            if ( Validator::filterValidate( \sanitize_text_field( \wp_unslash( $_POST['user'] ) ), 'int' ) ) {

                if ( User::deleteLoggedInDevice( $device, \sanitize_text_field( \wp_unslash( $_POST['user'] ) ) ) ) {
                    echo \json_encode( [ 'result' => 'success' ] );
                } else {
                    echo \json_encode( [ 'result' => 'failed' ] );
                }
            }

            die();
        }

        /**
         * AJAX request for deleting all the devices from the logged in devices for given user
         * It also can be used outside AJAX requests
         *
         * @since 1.0.0
         *
         * ! WP do_action sends first empty parameter if there are no parameters
         * TODO: extend this and the method above so it can recieves parameters as well
         * @param mixed $user - WP user for which remember me device must be deleted.
         *
         * @return void
         *
         * @SuppressWarnings(PHPMD.ExitExpressions)
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function deleteAllRememberMeDevices( $user = null ) {
            if ( ! isset( $_POST['nonce'] ) ||
                empty( $_POST['nonce'] ) ||
                ! \wp_verify_nonce( \sanitize_text_field( \wp_unslash( $_POST['nonce'] ) ), 'wp-secured-delete_all_device-ajax-nonce' ) ) {
                die();
            }

            if ( ! isset( $_POST['user'] ) || empty( $_POST['user'] ) ) {
                die();
            }
            if ( Validator::filterValidate( \sanitize_text_field( \wp_unslash( $_POST['user'] ) ), 'int' ) ) {
                if ( User::deleteAllLoggedInDevices( \sanitize_text_field( \wp_unslash( $_POST['user'] ) ) ) ) {
                    echo \json_encode( [ 'result' => 'success' ] ); // @codingStandardsIgnoreLine
                } else {
                    echo \json_encode( [ 'result' => 'failed' ] ); // @codingStandardsIgnoreLine
                }
            }

            die();
        }

        /**
         * Deletes QR code for the given user
         *
         * @return void
         *
         * @since 1.5
         *
         * @SuppressWarnings(PHPMD.ExitExpressions)
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function deleteQRCodeForUser() {
            if ( ! isset( $_POST['nonce'] ) ||
                empty( $_POST['nonce'] ) ||
                ! \wp_verify_nonce( \sanitize_text_field( \wp_unslash( $_POST['nonce'] ) ), 'wp-secured-wps_delete_qr-ajax-nonce' ) ) {
                die();
            }

            if ( ! isset( $_POST['user'] ) || empty( $_POST['user'] ) ) {
                die();
            }
            if ( Validator::filterValidate( \sanitize_text_field( \wp_unslash( $_POST['user'] ) ), 'int' ) ) {
                User::deleteUserTotp( \sanitize_text_field( \wp_unslash( $_POST['user'] ) ) );
                echo \json_encode( [ 'result' => 'success' ] );
            }

            die();
        }
    }
}
