<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Helpers;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use JShrink\Minifier;
use WPSEC\Helpers\ListFiles;

if ( ! class_exists( 'WPSEC\Helpers\JITJSCompiler' ) ) {
    /**
     * Just In Time SASS generator
     */
    class JITJSCompiler {

        /**
         * Holds the compiled JS file name
         *
         * @since 1.0.0
         *
         * @var string
         */
        public static $jsFileName = '';

        /**
         * Holds all the collected js files for compression and path
         *
         * @since 1.0.0
         *
         * @var array
         */
        private static $jsFiles = [];

        /**
         * Holds the checksum for all the source SASS files
         *
         * @see JITJSCompiler::getCheckSum for the used logic
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $checkSum = '';

        /**
         * Holds the compiled JS string
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $compiledJSString = '';

        /**
         * Collects the uncompressed JS checksum
         * Checks JS existence
         * Compiles new JS if necessary
         *
         * @since 1.0.0
         *
         * @return string full path to the compiled JS
         */
        public static function getJSFile(): string {
            self::getCheckSum();

            if ( ! self::checkFileExists() ) {
                foreach ( self::$jsFiles as $file ) {
                    self::$compiledJSString .= Minifier::minify( \file_get_contents( $file ) );
                }

                \file_put_contents( self::$jsFileName, self::$compiledJSString );
            }

            return self::$jsFileName;
        }

        /**
         * Checks if the JS file exists based on the collected checksum
         *
         * @since 1.0.0
         *
         * @return boolean
         */
        private static function checkFileExists(): bool {
            $uploads   = wp_upload_dir( null, false );
            $assetsDir = $uploads['basedir'] . WPSEC_PLUGIN_SECURED_ASSETS_DIR;

            if ( ! is_dir( $assetsDir ) ) {
                mkdir( $assetsDir, 0755, true );
            }

            self::$jsFileName = $assetsDir . \DIRECTORY_SEPARATOR . self::$checkSum . '.js';

            return \file_exists( self::$jsFileName );
        }

        /**
         * Generates checksum for the JS files
         * Current logic:
         * - get file size
         * - get file modification date
         * - sets string for all the collected files
         * - generates MD5 string
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function getCheckSum() {
            self::$jsFiles = ListFiles::getFiles( plugin_dir_path( WPSEC_PLUGIN_SECURED_FILENAME ) . 'assets/js-full/', 100, [], [ 'js' ] );

            $filesString = '';

            if ( ! empty( self::$jsFiles ) ) {
                foreach ( self::$jsFiles as $file ) {
                    $filesString .= \filesize( $file );
                    $filesString .= \filemtime( $file );
                }
            }

            self::$checkSum = md5( $filesString );
        }
    }
}
