<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Helpers;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use WPSEC\Helpers\ListFiles;
use ScssPhp\ScssPhp\Compiler;
use ScssPhp\ScssPhp\OutputStyle;

if ( ! class_exists( 'WPSEC\Helpers\JITSCSSCompiler' ) ) {
    /**
     * Just In Time SASS generator
     */
    class JITSCSSCompiler {

        /**
         * Holds the compiled CSS file name
         *
         * @since 1.0.0
         *
         * @var string
         */
        public static $cssFileName = '';

        /**
         * Holds all the collected scss files and path
         *
         * @since 1.0.0
         *
         * @var array
         */
        private static $scssFiles = [];

        /**
         * Holds the checksum for all the source SASS files
         *
         * @see JITSCSSCompiler::getCheckSum for the used logic
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $checkSum = '';

        /**
         * Holds the compiled SCSS string
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $compiledCSSString = '';

        /**
         * Collects the SASS checksum
         * Checks CSS existence
         * Compiles new CSS if necessary
         *
         * @since 1.0.0
         *
         * @return string full path to the compiled CSS
         */
        public static function getCssFile(): string {
            self::getCheckSum();

            if ( ! self::checkFileExists() ) {
                $scss = new Compiler();
                $scss->setOutputStyle( OutputStyle::COMPRESSED );
                foreach ( self::$scssFiles as $file ) {
                    self::$compiledCSSString .= $scss->compile( \file_get_contents( $file ) );
                }

                \file_put_contents( self::$cssFileName, self::$compiledCSSString );
            }

            return self::$cssFileName;
        }

        /**
         * Checks if the CSS file exists based on the collected checksum
         *
         * @since 1.0.0
         *
         * @return boolean
         */
        private static function checkFileExists(): bool {
            $uploads   = wp_upload_dir( null, false );
            $assetsDir = $uploads['basedir'] . WPSEC_PLUGIN_SECURED_ASSETS_DIR;

            if ( ! is_dir( $assetsDir ) ) {
                mkdir( $assetsDir, 0755, true );
            }

            self::$cssFileName = $assetsDir . \DIRECTORY_SEPARATOR . self::$checkSum . '.css';
            return \file_exists( self::$cssFileName );
        }

        /**
         * Generates checksum for the SASS files
         * Current logic:
         * - get file size
         * - get file modification date
         * - sets string for all the collected files
         * - generates MD5 string
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function getCheckSum() {
            self::$scssFiles = ListFiles::getFiles( plugin_dir_path( WPSEC_PLUGIN_SECURED_FILENAME ) . 'assets/sass/', 100, [], [ 'scss' ] );

            $filesString = '';

            if ( ! empty( self::$scssFiles ) ) {
                foreach ( self::$scssFiles as $file ) {
                    $filesString .= \filesize( $file );
                    $filesString .= \filemtime( $file );
                }
            }

            self::$checkSum = md5( $filesString );
        }
    }
}
