<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Helpers;

use OTPHP\TOTP;
use WPSEC\Controllers\User;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Helpers\OutOfBondEmail' ) ) {

    /**
     * Creates and send single sign in link to the user via email, and logs the user
     *
     * @since 1.0.0
     */
    class OutOfBondEmail {

        /**
         * Transient prefix used for storing the user OOB secret value
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $transientPrefix = '_wpsec_oob_';

        /**
         * Nonce name for the URL check
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $nonceName = '_nonce';

        /**
         * Nonce prefix for the URL check
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $nonceNamePrefix = '_wpsec_nonce_';

        /**
         * Sets the time period for how long the TOTP is valid
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $period = 300;

        /**
         * How many digits the password must be
         *
         * @since 1.0.0
         *
         * @var integer
         */
        private static $digits = 12;

        /**
         * Which algorithm is used for generating the proper TOTP
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $algorithm = 'sha1';

        /**
         * Used for AJAX calls, sends Out Of Bond email to the user with single link
         *
         * @since 1.0.0
         *
         * @return void
         *
         * @SuppressWarnings(PHPMD.Superglobals)
         */
        public static function sendOOBEmail() {

            $subject = __( 'One time login link', 'secured-wp' );

            if ( ! isset( $_GET['userId'] ) || empty( $_GET['userId'] ) ) {
                \wp_die( 1 );
            }

            if ( ! isset( $_GET['redirectTo'] ) || empty( $_GET['redirectTo'] ) ) {
                $redirectTo = \user_admin_url();
            } else {
                $redirectTo = \esc_url_raw( \sanitize_text_field( \wp_unslash( $_GET['redirectTo'] ) ) );
            }

            $userId  = (int) $_GET['userId'];
            $to      = User::getUser( $userId )->user_email;
            $headers = [ 'Content-Type: text/html; charset=UTF-8' ];

            $message = \sprintf(
                /* translators: %1$s: Name of the user, %2$s: Url of the site, %3$s: Number of the login attempts */
                __( 'Hello %1$s,<br>Use the following link to login to site %2$s.<br>If not you who is trying to login, please contact the administrator immediately.<br>Click on the following link to login: %3$s', 'ws-secured' ),
                User::getUser( $userId )->display_name,
                \get_bloginfo( 'name' ),
                self::generateLink( User::getUser( $userId )->ID, $redirectTo )
            );

            ob_start();
            WPHelper::getTemplatePart(
                'template-oob',
                [
                    'userName' => User::getUser( $userId )->display_name,
                    'blogName' => \get_bloginfo( 'name' ),
                    'oobLink'  => self::generateLink( User::getUser( $userId )->ID, $redirectTo ),
                ]
            );
            $message = ob_get_clean();

            $message = MailHelper::getMailHeader( __( 'Out of bond email link', 'secured-wp' ) ) . $message . MailHelper::getMailFooter();

            \wp_mail( $to, $subject, $message, $headers );

            echo \json_encode(
                [
                    'status'  => 'success',
                    'message' => \sprintf(
                        /* translators: %1$s: Seconds this link is valid */
                        __( 'Check your mail, you have %1$s seconds to use that link.', 'ws-secured' ),
                        self::$period
                    ),
                    'time'    => self::$period,
                ]
            );

            \wp_die();
        }

        /**
         * Validates the given password against the TOTP
         *
         * @since 1.0.0
         *
         * @param integer $userId - the WP user ID.
         * @param string  $pass - pass to check against.
         *
         * @return boolean
         */
        public static function validateUserOOB( int $userId, string $pass ): bool {
            $secret = \get_transient( self::$transientPrefix . $userId );

            if ( ! isset( $secret ) || empty( $secret ) ) {
                return false;
            }

            $otp = TOTP::create(
                $secret,
                self::$period,
                self::$algorithm,
                self::$digits
            );

            if ( $pass === $otp->at( time() ) ) {
                return true;
            }

            return false;
        }

        /**
         * Returns the transient prefix
         *
         * @since 1.0.0
         *
         * @return string
         */
        public static function getTransientPrefix() {
            return self::$transientPrefix;
        }

        /**
         * Deletes the transient for specific user
         *
         * @since 1.0.0
         *
         * @param integer $userId - The WP user ID.
         *
         * @return void
         */
        public static function deleteTransient( int $userId ) {
            \delete_transient( self::$transientPrefix . $userId );
        }

        /**
         * Returns the name of the nonce
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getNonceName(): string {
            return self::$nonceName;
        }

        /**
         * Returns the name prefix of the nonce
         *
         * @return string
         *
         * @since 1.0.0
         */
        public static function getNonceNamePrefix(): string {
            return self::$nonceNamePrefix;
        }

        /**
         * Generates OOB link which will be used in the email to the user
         *
         * @since 1.0.0
         *
         * @param integer $userId - the WP user ID.
         * @param string  $redirectTo - where to redirect - used in the link.
         *
         * @return string
         */
        private static function generateLink( int $userId, $redirectTo ): string {
            $link = '';

            $otp = TOTP::create(
                null, // Let the secret be defined by the class.
                self::$period,     // The period.
                self::$algorithm, // The digest algorithm.
                self::$digits      // The output will generate 6 digits.
            );

            \set_transient( self::$transientPrefix . $userId, $otp->getSecret(), self::$period );

            $params                     = [];
            $params['wps_otp']          = $otp->at( time() );
            $params['user_id']          = $userId;
            $params['redirect_to']      = $redirectTo;
            $params[ self::$nonceName ] = \wp_create_nonce( self::$nonceNamePrefix . $userId );

            $url = WPHelper::getSiteUrl();

            $link = \add_query_arg(
                $params,
                $url
            );

            return "<a href='$link'>" . __( 'Click', 'ws-secured' ) . '</a>';
        }
    }
}
