<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Helpers\PHPHelpers;

use RecursiveIteratorIterator;
use RecursiveDirectoryIterator;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Helpers\PHPHelpers\ClassHelper' ) ) {

    /**
     * General helper for php classes related logic
     *
     * Works only with PSR4
     *
     * @since 1.0.0
     */
    class ClassHelper {

        /**
         * Searches recursively through the given path for the all declared PHP classes
         *
         * @param string $namespace - Classes namespace.
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function findRecursive( string $namespace ): array {
            $namespacePath = self::translateNamespacePath( $namespace );

            if ( '' === $namespacePath ) {
                return [];
            }

            return self::searchClasses( $namespace, $namespacePath );
        }

        /**
         * Converts the directory to namespaces
         *
         * @param string $namespace - Classes namespace.
         *
         * @return string
         *
         * @since 1.0.0
         */
        protected static function translateNamespacePath( string $namespace ): string {
            $rootPath = WPSEC_PLUGIN_SECURED_PATH . 'classes' . DIRECTORY_SEPARATOR;

            $nsParts = explode( '\\', $namespace );
            array_shift( $nsParts );

            if ( empty( $nsParts ) ) {
                return '';
            }

            return realpath( $rootPath . implode( DIRECTORY_SEPARATOR, $nsParts ) ) ?? '';
        }

        /**
         * Returns the classes filtered by given namespace
         *
         * @param string $namespace - Classes namespace.
         * @param string $namespacePath - Classes namespace path.
         *
         * @return array
         *
         * @since 1.0.0
         */
        private static function searchClasses( string $namespace, string $namespacePath ): array {
            $classes = [];

            /**
             * Walks through dirs and searches for classes of given namespace
             *
             * @var \RecursiveDirectoryIterator $iterator
             * @var \SplFileInfo $item
             */
            foreach ( $iterator = new RecursiveIteratorIterator(
                new RecursiveDirectoryIterator( $namespacePath, RecursiveDirectoryIterator::SKIP_DOTS ),
                RecursiveIteratorIterator::SELF_FIRST
            ) as $item ) {
                if ( $item->isDir() ) {
                    $nextPath      = $iterator->current()->getPathname();
                    $nextNamespace = $namespace . '\\' . $item->getFilename();
                    $classes       = array_merge( $classes, self::searchClasses( $nextNamespace, $nextPath ) );
                    continue;
                }
                if ( $item->isFile() && $item->getExtension() === 'php' ) {
                    $class = $namespace . '\\' . $item->getBasename( '.php' );
                    if ( ! class_exists( $class ) ) {
                        continue;
                    }
                    $classes[] = $class;
                }
            }

            return $classes;
        }
    }
}
