<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Helpers;

use WPSEC\Validators\Validator;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Helpers\WPHelper' ) ) {
    /**
     * WP helpers function class
     *
     * @since 1.0.0
     */
    class WPHelper {

        /**
         * Deletes WP option by given name
         *
         * @since 1.0.0
         *
         * @param string $option - Option name.
         * @param string $networkId - WP network ID.
         * @param string $blogId - WP blog ID.
         *
         * @return void
         */
        public static function deleteOption(
            string $option,
            $networkId = '',
            $blogId = ''
            ) {

            if ( self::isMultisite() ) {
                \delete_blog_option( self::getBlogId( $blogId ), $option );
            } else {
                \delete_network_option( self::getNetworkId( $networkId ), $option );
            }
        }

        /**
         * Sets WP Option
         *
         * @since 1.0.0
         *
         * @param string $option - option name.
         * @param mixed  $value - value for the given option.
         * @param string $networkId - WP network ID.
         * @param string $blogId - WP blog ID.
         * @param bool   $global - is that global variable ? Multisite install.
         *
         * @return void
         */
        public static function setOption(
            string $option,
            $value,
            $networkId = '',
            $blogId = '',
            bool $global = false
            ) {

            if ( self::isMultisite() && ! $global ) {
                \update_blog_option( self::getBlogId( $blogId ), $option, $value );
            } else {
                \update_network_option( self::getNetworkId( $networkId ), $option, $value );
            }
        }

        /**
         * Returns option value
         *
         * @since 1.0.0
         *
         * @param string $option - Option name to extract value from.
         * @param mixed  $defaultValue - Default value tor return if option is not present.
         * @param string $networkId - Network Id to extract value from.
         * @param string $blogId - Blog Id to extract value from.
         * @param bool   $global - global setting or blog related setting flag.
         *
         * @return mixed
         */
        public static function getOption(
            string $option,
            $defaultValue = false,
            $networkId = '',
            $blogId = '',
            bool $global = false
            ) {

            if ( self::isMultisite() && ! $global ) {
                return \get_blog_option( self::getBlogId( $blogId ), $option, $defaultValue );
            } else {
                return \get_network_option( self::getNetworkId( $networkId ), $option, $defaultValue );
            }

            return false;
        }

        /**
         * Checks if current WP installation is multisite or not
         *
         * @since 1.0.0
         *
         * @return boolean
         */
        public static function isMultisite(): bool {
            if ( \is_multisite() ) {
                return true;
            }
            return false;
        }

        /**
         * Returns current blog ID
         *
         * @since 1.0.0
         *
         * @param string $blogId - WP blog ID.
         *
         * @return integer
         */
        public static function getBlogId( $blogId = '' ): int {

            if ( '' === $blogId && self::isMultisite() ) {
                $blogId = \get_current_blog_id();
            } else {
                if ( ! Validator::filterValidate( $blogId, 'int' ) ) {
                    $blogId = 0;
                }
            }

            return (int) ( ( $blogId ) ? $blogId : 0 );
        }

        /**
         * Returns current network ID
         *
         * @since 1.0.0
         *
         * @param string $networkId - WP network ID.
         *
         * @return mixed
         */
        public static function getNetworkId( $networkId = '' ) {

            if ( '' === $networkId && self::isMultisite() ) {
                $networkId = \get_current_network_id();
            } elseif ( ! Validator::filterValidate( $networkId, 'int' ) ) {
                $networkId = null;
            }

            return $networkId;
        }

        /**
         * Returns the current site URL
         *
         * @since 1.0.0
         *
         * @return string
         */
        public static function getSiteUrl() {
            $url = \get_site_url( \get_current_blog_id() );

            return $url;
        }

        /**
         * Returns all the sites in multisite install
         *
         * @param array $args - Array with the sites.
         *
         * @return array
         *
         * @since 1.0.0
         */
        public static function getSites( $args = [] ): array {
            return \get_sites( $args );
        }

        /**
         * Add support for $args to the template part
         *
         * @param string $templateSlug - the name of the template.
         * @param array  $args - arguments to pass to template.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function getTemplatePart( string $templateSlug, array $args = [] ) {

            if ( $args && is_array( $args ) ) {
                extract( $args ); // @codingStandardsIgnoreLine
            }

            $located = \locate_template( "{$templateSlug}.php" );

            if ( empty( $located ) ) {
                $located = WPSEC_PLUGIN_SECURED_PATH . \DIRECTORY_SEPARATOR . 'templates' . \DIRECTORY_SEPARATOR . "{$templateSlug}.php";
            }

            include $located;
        }

        /**
         * Is the current request an XML-RPC or REST request.
         *
         * @return bool
         */
        public static function isApiRequest(): bool {
            if ( defined( 'XMLRPC_REQUEST' ) && XMLRPC_REQUEST ) {
                return true;
            }

            if ( defined( 'REST_REQUEST' ) && REST_REQUEST ) {
                return true;
            }

            return false;
        }
    }
}
