<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use WPSEC\{
    Controllers\User,
    Controllers\Modules\TwoFASettings,
    Controllers\Modules\RememberMe,
    Controllers\Modules\XMLRPCprevents,
    Views\AdminSettingsView,
    Views\UsersList,
    Helpers\JITSCSSCompiler,
    Helpers\JITJSCompiler,
    Helpers\WPHelper,
    Helpers\AjaxRequests,
};
use WPSEC\Validators\Validator;

if ( ! class_exists( 'WPSEC\Secured' ) ) {
    /**
     * Base class for WPS secured
     *
     * @since 1.0.0
     */
    class Secured {

        /**
         * Holds the status of the delete data upon uninstall setting
         *
         * @since 1.0.0
         *
         * @var bool
         */
        private static $deleteUponUninstall = null;

        /**
         * Sets all the necessary hooks and creates the tables if needed
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function init() {
            self::initHooks();
        }

        /**
         * Adds settings link
         *
         * @since 1.0.0
         *
         * @param array $links - Gloabal links array.
         *
         * @return array
         */
        public static function addActionLinks( array $links ): array {
            $newLinks = [
                '<a href="' . \admin_url( 'options-general.php?page=' . WPSEC_PLUGIN_SECURED_SLUG ) . '">' . \esc_html__( 'Settings', 'secured-wp' ) . '</a>',
            ];
            return array_merge( $links, $newLinks );
        }

        /**
         * Checks for the minimum requirements and bails if they are not met
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function pluginActivation() {

            // Minimum PHP version check.
            if ( version_compare( phpversion(), WPSEC_REQUIRED_PHP_VERSION, '<' ) ) {

                // Plugin not activated info message.
                ?>
                <div class="update-nag">
                <?php
                echo \sprintf(
                /* translators: %1$s: PHP version */
                    \esc_html__( 'You need to update your PHP version to %1s.', 'secured-wp' ),
                    WPSEC_REQUIRED_PHP_VERSION // @codingStandardsIgnoreLine - that is defined constatnt no need to escape it
                );
                ?>
                <br />
                <?php echo \esc_html__( 'Actual version is:', 'secured-wp' ); ?> <strong><?php echo esc_html( phpversion() ); ?></strong>
                </div>
                <?php

                exit();
            }

            // Minimum WP version check.
            if ( version_compare( $GLOBALS['wp_version'], WPSEC_REQUIRED_WP_VERSION, '<' ) ) {

                // Plugin not activated info message.
                ?>
                <div class="update-nag">
                <?php
                echo \sprintf(
                /* translators: %1$s: WP version */
                    \esc_html__( 'You need to update your WP version to %1s.', 'secured-wp' ),
				WPSEC_REQUIRED_WP_VERSION // @codingStandardsIgnoreLine - that is defined constatnt no need to escape it
                );
                ?>
                <br />
                <?php echo \esc_html__( 'Actual version is:', 'secured-wp' ); ?> <strong><?php echo esc_html( $GLOBALS['wp_version'] ); ?></strong>
                </div>
                <?php

                exit;
            }
        }

        /**
         * Adds settings page to the admin menu
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function settingsPage() {
            \add_options_page(
                WPSEC_PLUGIN_SECURED_NAME . ' settings',
                WPSEC_PLUGIN_SECURED_NAME,
                'manage_options',
                WPSEC_PLUGIN_SECURED_SLUG,
                [
                    'WPSEC\\Secured',
                    'renderPluginSettingsPage',
                ]
            );
            \add_menu_page(
                WPSEC_PLUGIN_SECURED_NAME . ' settings',
                WPSEC_PLUGIN_SECURED_NAME,
                'manage_options',
                WPSEC_PLUGIN_SECURED_SLUG,
                [
                    'WPSEC\\Secured',
                    'renderPluginSettingsPage',
                ],
			'data:image/svg+xml;base64,' . \base64_encode( \file_get_contents( WPSEC_PLUGIN_SECURED_PATH . 'assets/images/the-logo.svg' ) ), // @codingStandardsIgnoreLine - :) well that one is necessary 
                81
            );
        }

        /**
         * Responsible for rendering the admin settings page
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function renderPluginSettingsPage() {
            AdminSettingsView::adminSettings();
        }

        /**
         * Returns the status of delete data upon uninstall
         *
         * @since 1.0.0
         *
         * @param mixed $blogId - WP blog ID.
         *
         * @return mixed
         */
        public static function isDeleteDataEnabled( $blogId = '' ) {
            if ( null === self::$deleteUponUninstall ) {
                self::$deleteUponUninstall =
                WPHelper::getOption(
                    WPSEC_PLUGIN_SECURED_DELETE_DATA_VAR_NAME,
                    WPSEC_PLUGIN_SECURED_DELETE_DATA_VAR_DEFAULT,
                    '',
                    $blogId,
                    true
                );

                self::$deleteUponUninstall = filter_var( self::$deleteUponUninstall, FILTER_VALIDATE_BOOLEAN );
            }

            return self::$deleteUponUninstall;
        }

        /**
         * Sets the value of the "delete on uninstall option"
         *
         * @param [type] $value - the value which needs to be deleted.
         * @param string $blogId - WP blog ID.
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function setDeleteOnUninstall( $value, $blogId = '' ) {
            if ( Validator::filterValidate( $value, 'bool' ) ) {
                WPHelper::setOption( WPSEC_PLUGIN_SECURED_DELETE_DATA_VAR_NAME, $value, '', $blogId, true );
            }
        }
        /**
         * Deletes data from the options
         *
         * @return void
         *
         * @since 1.0.0
         */
        public static function deleteData() {
            WPHelper::deleteOption( WPSEC_PLUGIN_SECURED_DELETE_DATA_VAR_NAME );
        }

        /**
         * Inits all the hooks the plugin will use
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function initHooks() {

            /**
             * Check users and login attempts - validates the login
             */
            \add_action( 'authenticate', [ 'WPSEC\\Controllers\\LoginCheck', 'check' ], 9999, 3 );

            /**
             * Validate OOB login
             */
            \add_action( 'login_form_confirm_oob', [ 'WPSEC\\Controllers\\LoginCheck', 'loginValidateOob' ], 1 );

            /**
             * Validates 2FA login from
             */
            \add_action( 'login_form_validate_2fa', [ 'WPSEC\\Controllers\\LoginCheck', 'loginValidateTwoFa' ] );
            if ( class_exists( 'WooCommerce' ) && ! (bool) \WPSEC\Controllers\Modules\TwoFASettings::getGlobalSettingsValue() ) {
                \add_action( 'wp_loaded', [ 'WPSEC\\Controllers\\LoginCheck', 'loginValidateTwoFa' ], 20 );
                /**
                 * Show QR code in the user profile section for WOOCOMMERCE
                 */
                if ( User::isCurrentlyLogged() && ! (bool) User::isTwoFaUserExcluded() ) {
                    \add_action(
                        'woocommerce_account_dashboard',
                        [ 'WPSEC\Views\UserProfile', 'editUserQR' ]
                    );
                }
            }

            if ( ! RememberMe::getGlobalSettingsValue() ) {
                \add_action( 'set_logged_in_cookie', [ 'WPSEC\\Controllers\\LoginCheck', 'setRememberMe' ] );
            }
            \add_action( 'wp_logout', [ 'WPSEC\\Controllers\\LoginCheck', 'logOut' ], 1 );

            if ( ! XMLRPCprevents::getGlobalSettingsValue() ) {
                add_filter( 'xmlrpc_enabled', '__return_false', 5 );
                remove_action( 'wp_head', 'rsd_link' );
                add_filter( 'bloginfo_url', [ 'WPSEC\\Controllers\\Modules\\XMLRPCprevents', 'removeHeaderPingLink' ], 11, 2 );
                add_filter( 'wp_headers', [ 'WPSEC\\Controllers\\Modules\\XMLRPCprevents', 'removeXmlrpcLink' ], PHP_INT_MAX );
                add_filter( 'xmlrpc_methods', [ 'WPSEC\\Controllers\\Modules\\XMLRPCprevents', 'removeXmlrpcMethods' ], PHP_INT_MAX );
            }

            /**
             * No user is logged in - add ajax hooks
             */
            if ( ! User::isCurrentlyLogged() && TwoFASettings::isOOBEnabled() ) {
                \wp_enqueue_script( 'jquery' );
                /**
                 * Adds ajax request for sending the OOB
                 */
                \add_action( 'wp_ajax_nopriv_send_oob', [ 'WPSEC\\Helpers\\OutOfBondEmail', 'sendOOBEmail' ] );
                \add_action( 'wp_ajax_send_oob', [ 'WPSEC\\Helpers\\OutOfBondEmail', 'sendOOBEmail' ] );
            }

            if ( is_admin() ) {
                // Settings Related actions.
                \add_action( 'admin_menu', [ 'WPSEC\\Secured', 'settingsPage' ] );

                /**
                 * Adds columns to the users listing table
                 * showing the status of every given user
                 */
                UsersList::init();

                self::addAssets();

                // UserProfile.
                global $pagenow;
                if ( 'profile.php' !== $pagenow || 'user-edit.php' !== $pagenow ) {
                    \add_action( 'show_user_profile', [ 'WPSEC\Views\UserProfile', 'userEditProfile' ] );
                    \add_action( 'edit_user_profile', [ 'WPSEC\Views\UserProfile', 'userEditProfile' ] );

                    // AJAX request for the user - do we need this globally for the Admin Part of the WP ?.
                    AjaxRequests::initAdmin();

                    \add_action( 'personal_options_update', [ 'WPSEC\Views\UserProfile', 'regenerateQRCode' ], 10, 1 );
                    \add_action( 'edit_user_profile_update', [ 'WPSEC\Views\UserProfile', 'regenerateQRCode' ], 10, 1 );
                }
            }

            /**
             * If user is logged in and there is woocommerce installed and the method is enabled,
             * we gona need the AJAX methods available.
             */
            if ( User::isCurrentlyLogged() ) {
                if ( class_exists( 'WooCommerce' ) ) {
                    if ( ! (bool) \WPSEC\Controllers\Modules\TwoFASettings::getGlobalSettingsValue() ) {
                        // AJAX request for the user - do we need this globally for the Admin Part of the WP ?.
                        AjaxRequests::initAdmin();
                    }
                }
            }
        }

        /**
         * Conditionally add assets only when they are necessary
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function addAssets() {
            if ( isset( $_REQUEST['page'] ) && WPSEC_PLUGIN_SECURED_SLUG === $_REQUEST['page'] ) {
                \wp_enqueue_style(
                    WPSEC_PLUGIN_SECURED_CSS_PREFIX . '_stylesheet',
                    \wp_upload_dir()['baseurl'] .
                    WPSEC_PLUGIN_SECURED_ASSETS_DIR . \DIRECTORY_SEPARATOR . \basename( JITSCSSCompiler::getCssFile() )
                );

                $data = 'var ' . WPSEC_PLUGIN_SECURED_JS_VAR . ' = "' . WPSEC_PLUGIN_SECURED_CSS_PREFIX . '"';

                \wp_enqueue_script(
                    WPSEC_PLUGIN_SECURED_CSS_PREFIX . '-admin-scripts',
                    \wp_upload_dir()['baseurl'] .
                    WPSEC_PLUGIN_SECURED_ASSETS_DIR . \DIRECTORY_SEPARATOR . \basename( JITJSCompiler::getJSFile() ),
                    [ 'jquery', 'jquery-ui-tooltip' ]
                );

                \wp_add_inline_script( WPSEC_PLUGIN_SECURED_CSS_PREFIX . '-admin-scripts', $data, 'before' );

                \wp_enqueue_script( 'jquery-ui-resizable' );
                \wp_enqueue_script( 'jquery-ui-tabs' );
            }
        }
    }
}
