<?php
/**
 * Plugin WPS secured
 *
 * @package   WPS
 * @author    wp-secured.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Validators;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

if ( ! class_exists( 'WPSEC\Helpers\Validator' ) ) {
    /**
     * Validator class - responsible for validating of different simple variables
     *
     * @since 1.0.0
     */
    class Validator {

        /**
         * Hold all the errors collected through the validating process
         *
         * @since 1.0.0
         *
         * @var array
         */
        private static $errors = [];

        /**
         * Name of the variable checked for validation
         *
         * @since 1.0.0
         *
         * @var string
         */
        private static $variableKey = '';

        /**
         * Validates given array with rules
         *
         * @since 1.0.0
         *
         * @param mixed $vars - vars that need to be checked.
         * @param array $rules - rules to be applied.
         *
         * @return bool
         */
        public static function validate( $vars, $rules = [] ): bool {
            if ( ! is_array( $vars ) ) {
                $vars = [ $vars ];
            }

            foreach ( $vars as $variable ) {
                $valid = true;
                self::variableAsKey( $variable );
                self::$errors[ self::$variableKey ] = ''; // init variable.

                if ( empty( $rules ) ) {
                    $valid                              = false;
                    self::$errors[ self::$variableKey ] = __( 'No rules are set - nothing to test against', 'secured-wp' );
                }
                foreach ( $rules as $rule => $params ) {
                    $rule = strval( $rule );
                    switch ( $rule ) {
                        case 'required':
                            if ( ! isset( $variable ) ) {
                                $valid                              = false;
                                self::$errors[ self::$variableKey ] = __( 'Variable is not set', 'secured-wp' );
                            }
                            break;
                        case 'positiveInt':
                            $valid = self::validateNumber( $variable, 1 );
                            break;
                        case 'intRange':
                            $min = false;
                            $max = false;
                            if ( isset( $params['min'] ) ) {
                                $min = $params['min'];
                            }
                            if ( isset( $params['max'] ) ) {
                                $max = $params['max'];
                            }
                            $valid = self::validateNumber( $variable, $min, $max );
                            break;
                        case 'email':
                            $valid = self::validateEmail( $variable );
                            break;
                        default:
                            $valid                              = false;
                            self::$errors[ self::$variableKey ] = __( 'No rules are set - nothing to test against', 'secured-wp' );
                            break;
                    }
                }
            }
            return $valid;
        }

        /**
         * Returns array with the errors collected (if any)
         *
         * @since 1.0.0
         *
         * @return array
         */
        public static function getErrors(): array {
            return self::$errors;
        }

        /**
         * Validates email
         *
         * @since 1.0.0
         *
         * @param string $variable - mail string to be validated.
         *
         * @return bool
         */
        public static function validateEmail( $variable ): bool {
            $valid = true;

            if ( false === ( $valid = self::filterValidate( $variable, 'email' ) ) ) {
                self::$errors[ self::$variableKey ] .= __( 'Variable is not valid e-mail', 'secured-wp' ) . "\n";
                $valid                               = false;
            }
            return $valid;
        }

        /**
         * Uses standard PHP filter validation
         *
         * @since 1.0.0
         *
         * @param mixed $variable - variable that needs to be checked.
         * @param mixed $type - type to be checked against.
         *
         * @return bool
         */
        public static function filterValidate( $variable, $type ): bool {
            $result = false;

            switch ( $type ) {
                case 'email':
                    $result = (bool) filter_var( $variable, FILTER_VALIDATE_EMAIL );
                    break;

                case 'int':
                    if ( false !== ( $result = filter_var( $variable, FILTER_VALIDATE_INT ) ) ) {
                        $result = true;
                    }
                    break;

                case 'bool':
                    if ( null !== filter_var( $variable, FILTER_VALIDATE_BOOLEAN, FILTER_NULL_ON_FAILURE ) ) {
                        $result = true;
                    }
                    break;

                default:
                    break;
            }

            return $result;
        }

        /**
         * Validates numbers
         *
         * @since 1.0.0
         *
         * @param type      $variable - variable that needs to be checked.
         * @param type|bool $min - minimul value.
         * @param type|bool $max - maximum value.
         *
         * @return bool
         */
        public static function validateNumber( $variable, $min = false, $max = false ): bool {
            $valid         = true;
            $filterOptions = [ 'options' => [] ];
            if ( $min ) {
                $filterOptions['options']['min_range'] = $min;
            }
            if ( $max ) {
                $filterOptions['options']['max_range'] = $max;
            }
            if ( ! is_numeric( $variable ) ) {
                $valid                              = false;
                self::$errors[ self::$variableKey ] = __( 'Variable is not integer', 'secured-wp' );
            } else {
                if ( is_string( $variable ) ) {
                    if ( false === ctype_digit( $variable ) ) {
                        self::$errors[ self::$variableKey ] .= __( 'Variable is not positive integer!', 'secured-wp' ) . "\n";
                        $valid                               = false;
                    }

                    if ( $valid ) {
                        /*
                         *  It is valid but it is string, casting gives the
                         *  opportunity to validate the range using filter_var
                         */
                        $variable = (int) $variable;
                    }
                }
            }
            if ( $valid && false === filter_var( $variable, FILTER_VALIDATE_INT, $filterOptions ) ) {
                $valid = false;
                if ( $min > 0 && false === $max ) {
                    self::$errors[ self::$variableKey ] .= __( 'Variable is not positive integer!', 'secured-wp' ) . "\n";
                }
                if ( false !== $max ) {
                    self::$errors[ self::$variableKey ] .= __( 'Variable is not in given range', 'secured-wp' ) . "\n";
                }
            }
            return $valid;
        }

        /**
         * You can not use integers as key without breaking the PHP arrays most of the time
         *
         * @since 1.0.0
         *
         * @param type $variable - variable usualy int that needs to be converted to string.
         *
         * @return void
         */
        private static function variableAsKey( $variable ): void {
            self::$variableKey = '_' . $variable;
        }
    }
}
