<?php
/**
 * Plugin WPS security
 *
 * @package   WPS
 * @author    wp-security.com
 * @copyright Copyright © 2021
 */

declare(strict_types=1);

namespace WPSEC\Views;

defined( 'ABSPATH' ) || exit; // Exit if accessed directly.

use OTPHP\TOTP;
use WPSEC\Controllers\{
    User,
    Modules\RememberMe,
};

if ( ! class_exists( 'WPSEC\Views\UserProfile' ) ) {

    /**
     * Responsible for users profile page
     *
     * @since 1.0.0
     */
    class UserProfile {

        /**
         * QR code regeneration nonce prefix
         *
         * @var string
         *
         * @since 1.2
         */
        private static $qrNoncePrefix = '_regenerate-qr-code_';

        /**
         * Shows user profile section
         *
         * @since 1.0.0
         *
         * @param mixed $user - the WP user.
         * @param array $additionalArgs - Additional args - not in use currently.
         *
         * @return void
         */
        public static function userEditProfile( $user, $additionalArgs = [] ) {

            $devices = User::getLoggedInDevices( $user );
            ?>
                <h2><?php echo \esc_html__( 'Secured WP - Options', 'secured-wp' ); ?></h2>
                <h3><?php echo \esc_html__( 'Current devices with "Remember Me" option enabled', 'secured-wp' ); ?></h3>
                <table class="wp-secured-logged-devices widefat striped table-view-list">
                    <thead>
                    <tr>
                        <th scope="col" class="wp-secured-device manage-column column-device column-primary">
                        <?php echo \esc_html__( 'Device', 'secured-wp' ); ?>
                        </th>
                        <th scope="col" class="wp-secured-device manage-column column-device column-primary">
                        <?php echo \esc_html__( 'Expires', 'secured-wp' ); ?>
                        </th>
                        <th scope="col" class="wp-secured-remove manage-column column-remove">
                        <?php echo \esc_html__( 'Remove', 'secured-wp' ); ?>
                        </th> 
                    </tr>
                    </thead>

                    <tbody id="the-list">
                <?php
                if ( empty( $devices ) ) {
                    ?>
                        <tr class="no-items">
                            <td class="colspanchange" colspan="3"><?php echo \esc_html__( 'No logged in devices found', 'secured-wp' ); ?></td>
                        </tr>
                        <?php
                } else {
                    foreach ( $devices as $device ) {
                        ?>
                        <tr class="wp-secured-device-row">
                            <td>
                            <?php echo \esc_html( $device ); ?>
                            </td>
                            <td>
                            <?php
                            $expTime = RememberMe::getExpireTime( $device, $user->ID );
                            if ( $expTime < time() ) {
                                echo \esc_html__( 'Expired', 'secured-wp' );
                            } else {
                                echo \esc_html( \date_i18n( \get_option( 'date_format' ) . ', ' . \get_option( 'time_format' ), RememberMe::getExpireTime( $device, $user->ID ) ) );
                            }
                            ?>
                            </td>
                            <td>
                            <?php
                            \submit_button(
                                __( 'Remove', 'secured-wp' ),
                                'delete',
                                'remove-device-password-' . md5( $device ),
                                false,
                                [
                                    /* translators: %s: the application password's given name. */
                                    'aria-label'  => sprintf( \esc_html__( 'Remove "%s"' ), \esc_html( $device ) ),
                                    'data-device' => \base64_encode( $device ),
                                ]
                            );
                            ?>

                            </td>
                        </tr>
                            <?php
                    }
                }
                ?>
                    </tbody>

                    <tfoot>
                    <tr>
                        <th scope="col" class="wp-secured-device manage-column column-device column-primary">
                        <?php echo \esc_html__( 'Device', 'secured-wp' ); ?>
                        </th>
                        <th scope="col" class="wp-secured-device manage-column column-device column-primary">
                        <?php echo \esc_html__( 'Expires', 'secured-wp' ); ?>
                        </th>
                        <th scope="col" class="wp-secured-remove manage-column column-remove">
                        <?php echo \esc_html__( 'Remove', 'secured-wp' ); ?>
                        </th> 
                    </tr>
                    </tfoot>

                </table>
                <div class="tablenav">
                    <div class="alignright">
                    <?php
                    \submit_button(
                        __( 'Remove all logged in devices', 'secured-wp' ),
                        'delete',
                        'remove-all-logged-devices',
                        false
                    );
                    ?>
                    </div>
                </div>
                <hr>
                <?php
                if ( \current_user_can( 'edit_user', User::getUser()->ID ) && ! (bool) User::isTwoFaUserExcluded() ) {
                    self::editUserQR();
                }
                ?>
                <?php
                self::deleteDeviceJS();
                self::deleteAllDeviceJS();
        }

        /**
         * Shows the QR code for the given user in the setting
         *
         * @since 1.0.0
         *
         * @return void
         */
        public static function editUserQR() {
            if ( ! (bool) \WPSEC\Controllers\Modules\TwoFASettings::getGlobalSettingsValue() ) {
                ?>
            <h3><?php echo \esc_html__( 'QR code for the TOTP 2FA login', 'secured-wp' ); ?></h3>
                <?php
                $otp = TOTP::create( User::getUserTotp() );

                $otp->setLabel(
                    \get_bloginfo( 'name' )
                );
                $grCodeUri = $otp->getQrCodeUri(
                    'https://api.qrserver.com/v1/create-qr-code/?data=[DATA]&size=200x200&ecc=M',
                    '[DATA]'
                );
                ?>
            <div style='width:100%;margin: 0 auto; text-align:center;'><img src='<?php echo \esc_attr( $grCodeUri ); ?>'></div>
            <div><?php echo \esc_html__( 'Or use the following key, by entering it directly in you preferable authentication application:', 'secured-wp' ); ?></div>
            <div><strong><?php echo \esc_html( $otp->getSecret() ); ?></strong></div>
            <div class="tablenav">
                <div class="alignleft">
                <?php
                $nonce = \wp_create_nonce( self::$qrNoncePrefix . User::getUser()->ID );

                ?>
                <input type="hidden" name="qr-nonce" value="<?php echo \esc_attr( $nonce ); ?>" />
                <?php

                if ( function_exists( 'submit_button' ) ) {
                    \submit_button(
                        \__( 'Regenerate QR code', 'secured-wp' ),
                        'delete',
                        'regenerate-qr-code',
                        false
                    );
                } else {
                    ?>
                <input type="submit" name="regenerate-qr-code" id="regenerate-qr-code" class="button delete" value="Regenerate QR code">
                    <?php
                    self::deleteQRCodeJS();
                }
                ?>
                </div>
            </div>
                <?php
            }
        }

        /**
         * Deletes the TOTP code for the user, it will be regenerated next time the user logs / see its profile page
         *
         * @param int $userId - the user which TOTP must be regenerated.
         *
         * @return void
         *
         * @since 1.3
         */
        public static function regenerateQRCode( $userId ) {
            if ( \current_user_can( 'edit_user', $userId ) ) {
                if ( isset( $_POST['qr-nonce'] ) &&
                \wp_verify_nonce( \sanitize_text_field( \wp_unslash( $_POST['qr-nonce'] ) ), self::$qrNoncePrefix . $userId ) &&
                isset( $_POST['regenerate-qr-code'] ) ) {

                    User::deleteUserTotp( $userId );
                }
            }
        }

        /**
         * Javascript for single logged in device deletion
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function deleteDeviceJS() {
            ?>
        <script>
            ( function( jQuery ) {
                jQuery('.wp-secured-logged-devices').on('click', '.delete', function(e) {
                    e.preventDefault();
                    var deleteData = {
                        'action': 'logged_device_delete',
                        'nonce': '<?php echo \esc_attr( \wp_create_nonce( 'wp-secured-delete_device-ajax-nonce' ) ); ?>',
                        'user': '<?php echo \esc_attr( User::getUser()->ID ); ?>',
                        'device': jQuery(this).data('device')      // We pass php values differently!
                    };

                    let that = this;

                    jQuery.ajax({
                        type: "post",
                        dataType: "json",
                        url: ajaxurl,
                        data: deleteData,
                        success: function(msg) {
                            if ( 'success' == msg['result'] ) {
                                jQuery( that ).parents( "tr" ).fadeOut();
                            }
                        }
                    });
                });
            }( jQuery ) );
        </script>
            <?php
        }

        /**
         * Javascript for all logged in device deletion
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function deleteAllDeviceJS() {
            ?>
        <script>
            ( function( jQuery ) {
                jQuery('.alignright').on('click', '#remove-all-logged-devices', function(e) {
                    e.preventDefault();
                    var deleteData = {
                        'action': 'all_logged_device_delete',
                        'user': '<?php echo \esc_attr( User::getUser()->ID ); ?>',
                        'nonce': '<?php echo \esc_attr( \wp_create_nonce( 'wp-secured-delete_all_device-ajax-nonce' ) ); ?>',
                    };

                    let that = this;

                    jQuery.ajax({
                        type: "post",
                        dataType: "json",
                        url: ajaxurl,
                        data: deleteData,
                        success: function(msg) {
                            if ( 'success' == msg['result'] ) {
                                jQuery( '.wp-secured-logged-devices tbody' ).fadeOut();
                            }
                        }
                    });
                });
            }( jQuery ) );
        </script>
            <?php
        }

        /**
         * Javascript for qr code deletion
         *
         * @since 1.0.0
         *
         * @return void
         */
        private static function deleteQRCodeJS() {
            ?>
        <script>
            ( function( jQuery ) {
                jQuery('.alignleft').on('click', '#regenerate-qr-code', function(e) {
                    e.preventDefault();
                    var ajaxurl = "<?php echo \esc_url( admin_url( 'admin-ajax.php' ) ); ?>"
                    var deleteData = {
                        'action': 'wps_delete_qr',
                        'user': '<?php echo \esc_attr( User::getUser()->ID ); ?>',
                        'nonce': '<?php echo \esc_attr( \wp_create_nonce( 'wp-secured-wps_delete_qr-ajax-nonce' ) ); ?>',
                    };

                    let that = this;

                    jQuery.ajax({
                        type: "post",
                        dataType: "json",
                        url: ajaxurl,
                        data: deleteData,
                        success: function( msg ) {
                            if ( 'success' == msg['result'] ) {
                                location.reload(); ;
                            }
                        }
                    });
                });
            }( jQuery ) );
        </script>
            <?php
        }
    }
}
